with System;
with Ada.Text_IO;
with Sem;
with Canon;
with Disp_Tree;
with Errorout; use Errorout;
with Flags;
with Disp_Vhdl;
with Std_Package;
with Translation;
with Types;
with Iirs_Utils;

package body Back_End is
   procedure Finish_Compilation
     (Unit : Iir_Design_Unit; Main : Boolean := False)
   is
      use Ada.Text_IO;
      Lib : Iir;
   begin
      --  No need to semantize during elaboration.
      --if Flags.Will_Elaborate then
      --   return;
      --end if;

      Lib := Get_Library_Unit (Unit);

      if (Main or Flags.Dump_All) and then Flags.Dump_Parse then
         Disp_Tree.Disp_Tree (Unit);
      end if;

      --  Semantic analysis.
      if Flags.Verbose then
         Put_Line ("semantize " & Disp_Node (Lib));
      end if;
      Sem.Semantic (Unit);

      if (Main or Flags.Dump_All) and then Flags.Dump_Sem then
         Disp_Tree.Disp_Tree (Unit);
      end if;

      if Errorout.Nbr_Errors > 0 then
         raise Compilation_Error;
      end if;

      if (Main or Flags.List_All) and then Flags.List_Sem then
         Disp_Vhdl.Disp_Vhdl (Unit);
      end if;

      --  Canonalisation.
      ------------------
      if Flags.Verbose then
         Put_Line ("canonicalize " & Disp_Node (Lib));
      end if;

      Canon.Canonicalize (Unit);

      if (Main or Flags.Dump_All) and then Flags.Dump_Canon then
         Disp_Tree.Disp_Tree (Unit);
      end if;

      if Errorout.Nbr_Errors > 0 then
         raise Compilation_Error;
      end if;

      if (Main or Flags.List_All) and then Flags.List_Canon then
         Disp_Vhdl.Disp_Vhdl (Unit);
      end if;

      if Translation.Flag_Elaborate then
         if Get_Kind (Lib) = Iir_Kind_Architecture_Declaration then
            declare
               Config : Iir_Design_Unit;
            begin
               Config := Canon.Create_Default_Configuration_Declaration (Lib);
               Set_Default_Configuration_Declaration (Lib, Config);
               if (Main or Flags.Dump_All) and then Flags.Dump_Canon then
                  Disp_Tree.Disp_Tree (Config);
               end if;
               if (Main or Flags.List_All) and then Flags.List_Canon then
                  Disp_Vhdl.Disp_Vhdl (Config);
               end if;
            end;
         end if;
         return;
      end if;

      --  Translation
      ---------------
      if not Main then
         --  Main units (those from the analyzed design file) are translated
         --  directly by ortho_front.
         if Flags.Verbose then
            Put_Line ("translate " & Disp_Node (Lib));
         end if;

         Translation.Translate (Unit, Main);

         if Errorout.Nbr_Errors > 0 then
            raise Compilation_Error;
         end if;
      end if;

   end Finish_Compilation;

   procedure Internal_Compilation is
   begin
      Translation.Initialize;
      Canon.Canon_Add_Labels := True;
      --Translation.Translate (Std_Package.Std_Standard_Unit, False);
      --Translation.Save_Std_Package_Node;
      if Flags.List_All and then Flags.List_Annotate then
         Disp_Vhdl.Disp_Vhdl (Std_Package.Std_Standard_Unit);
      end if;
   end Internal_Compilation;

   procedure Disp_Vhdl_Info (Node : Iir) is
      use System;
   begin
      if Get_Back_End_Info (Node) = Null_Address then
         return;
      end if;
--      Annotations.Disp_Vhdl_Info (Node);
   end Disp_Vhdl_Info;

   procedure Disp_Tree_Info (Node : Iir) is
   begin
      --Translation.Disp_Tree_Info (Node);
      Ada.Text_IO.Put_Line ("present");
   end Disp_Tree_Info;

      -- Transform a library identifier into a file name.
   -- Very simple mechanism: just add '-simVV.cf' extension, where VV
   -- is the version.
   function Library_To_File_Name (Library: Iir_Library_Declaration)
     return String
   is
      use Types;
   begin
      case Flags.Vhdl_Std is
         when Vhdl_87 =>
            return Iirs_Utils.Get_String (Library) & "-obj87.cf";
         when Vhdl_93c | Vhdl_93 | Vhdl_98 =>
            return Iirs_Utils.Get_String (Library) & "-obj93.cf";
      end case;
   end Library_To_File_Name;
end Back_End;
