--  Command line flags.
--  All the variables declared in this package are set by Parse_Option function
--  and can by read as soon as the command line is parsed.
--
--  Since the names are not prefixed, this package is expected to be with'ed
--  but not to be use'd.

with Types; use Types;

package Flags is
   -- Standard accepted.
   Vhdl_Std: Vhdl_Std_Type := Vhdl_93c;

   -- Return true if opt is recognize by flags.
   --  Note: std_names.std_names_initialize and files_map.init_pathes must have
   --  been called before this subprogram.
   function Parse_Option (Opt: String) return Boolean;

   -- Disp help about these options.
   procedure Disp_Options_Help;

   -- If set, a multi-bytes sequence can appear in a comment, ie, all
   -- characters except VT, CR, LF and FF are allowed in a comment.
   -- Set by -C and --mb-comments
   Mb_Comment: Boolean := False;

   -- If set, relax rules about std library: working library can be std.
   Bootstrap : Boolean := False;

   -- If set, be verbose during the research of an identifier.
   -- Set by --trace-find
   Trace_Find: Boolean := False;

   -- If set, disp actions done during simulation.
   -- Set by --trace-simulation
   Trace_Simulation: Boolean := False;

   -- If set, disp net update during simulation
   Trace_Update: Boolean := False;

   --  If set, ligne number of statements is displayed before execution
   --  Set by --trace-stmt
   Trace_Statements : Boolean := False;

   -- If set, disp elaboration informations.
   Trace_Elaboration: Boolean := False;

   -- If set, disp how drivers are created.
   Trace_Drivers : Boolean := False;

   -- If set, disp annotations informations.
   Trace_Annotation: Boolean := False;

   -- If set, disp top/all signals after each non-delta cycle.
   Trace_Top_Signals: Boolean := False;
   Trace_All_Signals: Boolean := False;

   -- If set, show the propagation
   Trace_Net : Boolean := False;

   -- If set, show a message before a new component instanciation
   -- Set by --disp-link.
   Disp_Link: Boolean := False;

   -- If set, disp all the signals before starting simulation.
   Disp_Signals: Boolean := False;

   -- If set, disp variables (and constants) after each delta.
   Disp_Objects: Boolean := False;

   -- Disp the instance tree.
   Disp_Tree: Boolean := False;

   --  Options -dX
   --  -dp: disp tree after parsing
   Dump_Parse: Boolean := False;

   --  -ds: disp tree after semantic
   Dump_Sem: Boolean := False;

   --  -dc: disp tree after canon
   Dump_Canon : Boolean := False;

   --  -da: disp tree after annotation
   Dump_Annotate: Boolean := False;

   --  --dall: makes -dX options to apply to all files.
   Dump_All: Boolean := False;

   -- -lX options: list tree as a vhdl file.

   -- --lall option: makes -lX options to apply to all files
   List_All: Boolean := False;

   -- -lv: list verbose
   List_Verbose: Boolean := False;

   -- -ls: list tree after semantic.
   List_Sem: Boolean := False;

   -- -lc: list tree after canon.
   List_Canon: Boolean := False;

   -- -la: list tree after back-end annotation.
   List_Annotate: Boolean := False;

   -- -v: disp phase of compilation.
   Verbose : Boolean := False;

   --  If set to true, it means that anything analysed will be used for
   --  elaboration.
   --  The purpose is to avoid spurious warning "will be checked
   --  at elaboration"
   --Will_Elaborate : Boolean := False;

   -- --time-resolution=X
   -- Where X corresponds to:
   -- fs => 'f'
   -- ps => 'p'
   -- ns => 'n'
   -- us => 'u'
   -- ms => 'm'
   -- sec => 's'
   -- min => 'M'
   -- hr => 'h'
   Time_Resolution: Character := 'f';

   -- --assert-level=X
   -- If an assertion fails with a level >= X, then execution stop.
   -- X can be note, warning, error, failure, none.
   Assert_Level : Natural := 3;

   -- --warn-undriven
   Warn_Undriven : Boolean := False;

   --  --warn-binding
   --  Should emit a warning when there is no default binding for a component
   --  instantiation.
   Warn_Default_Binding : Boolean := False;
end Flags;
