with Grt.Types; use Grt.Types;
with Grt.Typedesc; use Grt.Typedesc;

--  The design hierarchy names are created by this package.
package Grt.Hierarchy is
   pragma Preelaborate (Grt.Hierarchy);

   --  Note: this enumeration is also defined in the translator, and both
   --  definitions must correspond.
   type Ghdl_Name_Kind is
     (Ghdl_Name_Library,
      Ghdl_Name_Entity,
      Ghdl_Name_Architecture,
      Ghdl_Name_Block,
      Ghdl_Name_Generate_Iterative,
      Ghdl_Name_Generate_Conditional,
      Ghdl_Name_Component,
      Ghdl_Name_Package,
      Ghdl_Name_Process,
      Ghdl_Name_Configuration,
      Ghdl_Name_Signal,
      Ghdl_Name_Top);
   pragma Convention (C, Ghdl_Name_Kind);

   --  Name of an instance.
   --  Note: the layout of this record corresponds to the layout defined in
   --  the translator.
   type Ghdl_Instance_Name_Type;
   type Ghdl_Instance_Name_Acc is access Ghdl_Instance_Name_Type;
   type Ghdl_Instance_Name_Type (Kind : Ghdl_Name_Kind) is record
      Name : Ghdl_Str_Len_Ptr;
      Parent : Ghdl_Instance_Name_Acc;
      Brother : Ghdl_Instance_Name_Acc;
      case Kind is
         when Ghdl_Name_Entity
           | Ghdl_Name_Architecture
           | Ghdl_Name_Block
           | Ghdl_Name_Generate_Iterative
           | Ghdl_Name_Generate_Conditional
           | Ghdl_Name_Component
           | Ghdl_Name_Package
           | Ghdl_Name_Top =>
           Child : Ghdl_Instance_Name_Acc;
           case Kind is
              when Ghdl_Name_Generate_Iterative =>
                 Iter_Typedesc : Ghdl_Desc_Ptr;
                 Iter_Addr : Ghdl_Value_Ptr;
              when others =>
                 null;
           end case;
         when Ghdl_Name_Signal =>
            Sig_Mode : Mode_Signal_Type;
            Sig_Kind : Kind_Signal_Type;
            Sig_Indexes : Sig_Table_Range;
            Sig_Type_Desc : Ghdl_Desc_Ptr;
         when Ghdl_Name_Library
           | Ghdl_Name_Process
           | Ghdl_Name_Configuration =>
            null;
      end case;
   end record;

   --  Design hierarchy top entity.
   Top : aliased Ghdl_Instance_Name_Type := (Kind => Ghdl_Name_Top,
                                             Name => null,
                                             Parent => null,
                                             Brother => null,
                                             Child => null);
private
   pragma Export (C, Top, "__ghdl_hierarchy_top");
end Grt.Hierarchy;

