with System;
with Ada.Unchecked_Conversion;
with Interfaces; use Interfaces;

package Grt.Types is
   pragma Preelaborate (Grt.Types);

   type Ghdl_Ptr is new System.Address;
   type Ghdl_Index_Type is mod 2 ** System.Word_Size;
   type Ghdl_Integer is new Integer_32;
   subtype Ghdl_Real is Long_Float;

   type Ghdl_Dir_Type is (Dir_To, Dir_Downto);
   for Ghdl_Dir_Type use (Dir_To => 0, Dir_Downto => 1);
   for Ghdl_Dir_Type'Size use 8;

   subtype Std_Integer is Ghdl_Integer;

   type Std_Time is new Integer_64;
   Bad_Time : constant Std_Time := Std_Time'First;

   type Std_Integer_Trt is record
      Left : Std_Integer;
      Right : Std_Integer;
      Dir : Ghdl_Dir_Type;
      Length : Ghdl_Index_Type;
   end record;

   subtype Std_Character is Character;
   type Std_String_Base is array (Ghdl_Index_Type) of Std_Character;
   type Std_String_Basep is access Std_String_Base;

   type Std_String_Bound is record
      Dim_1 : Std_Integer_Trt;
   end record;
   type Std_String_Boundp is access Std_String_Bound;

   type Std_String is record
      Base : Std_String_Basep;
      Bounds : Std_String_Boundp;
   end record;

   --  Same as STD_STRING but with address.  Used to create string.
   type Std_String_Address is record
      Base : System.Address;
      Bounds : System.Address;
   end record;

   type Std_String_Ptr is access Std_String;

   --  Str_len.
   type String_Ptr is access String (1 .. Natural'Last);
   type Ghdl_Str_Len_Type is record
      Len : Natural;
      Str : String_Ptr;
   end record;
   --  Same as previous one, but using 'address.
   type Ghdl_Str_Len_Address_Type is record
      Len : Natural;
      Str : System.Address;
   end record;
   type Ghdl_Str_Len_Ptr is access constant Ghdl_Str_Len_Type;
   type Ghdl_Str_Len_Array is array (Natural) of Ghdl_Str_Len_Type;
   type Ghdl_Str_Len_Array_Ptr is access all Ghdl_Str_Len_Array;

   --  Location is used for errors/messages.
   type Ghdl_Location is record
      Filename : Ghdl_Str_Len_Ptr;
      Line : Integer;
      Col : Integer;
   end record;
   type Ghdl_Location_Ptr is access Ghdl_Location;

   --  Identifier for a process.
   type Process_Id is new Integer;

   --  Signal index.
   type Sig_Table_Index is new Integer;

   --  A range of signals.
   type Sig_Table_Range is record
      First, Last : Sig_Table_Index;
   end record;

   --  Simple values, used for signals.
   type Mode_Type is
     (Mode_B2, Mode_E8, Mode_E32, Mode_I32, Mode_P64, Mode_F64);

   type Ghdl_B2 is new Boolean;
   type Ghdl_E8 is new Unsigned_8;
   type Ghdl_E32 is new Unsigned_32;
   type Ghdl_I32 is new Integer_32;
   type Ghdl_P64 is new Integer_64;
   type Ghdl_F64 is new IEEE_Float_64;

   type Ghdl_B2_Array is array (Ghdl_Index_Type range <>) of Ghdl_B2;
   type Ghdl_E8_Array is array (Ghdl_Index_Type range <>) of Ghdl_E8;
   type Ghdl_I32_Array is array (Ghdl_Index_Type range <>) of Ghdl_I32;

   type Value_Union (Mode : Mode_Type := Mode_B2) is record
      case Mode is
         when Mode_B2 =>
            B2 : Ghdl_B2;
         when Mode_E8 =>
            E8 : Ghdl_E8;
         when Mode_E32 =>
            E32 : Ghdl_E32;
         when Mode_I32 =>
            I32 : Ghdl_I32;
         when Mode_P64 =>
            P64 : Ghdl_P64;
         when Mode_F64 =>
            F64 : Ghdl_F64;
      end case;
   end record;
   pragma Unchecked_Union (Value_Union);

   type Ghdl_Value_Ptr is access Value_Union;
   function To_Ghdl_Value_Ptr is new Ada.Unchecked_Conversion
     (Source => System.Address, Target => Ghdl_Value_Ptr);

   --  Mode of a signal.
   type Mode_Signal_Type is
     (Mode_Signal,
      Mode_In, Mode_Out, Mode_Inout, Mode_Buffer, Mode_Linkage,
      Mode_Stable, Mode_Quiet, Mode_Guard,
      Mode_Conv_In, Mode_Conv_Out);
   pragma Convention (C, Mode_Signal_Type);

   subtype Mode_Signal_Port is
     Mode_Signal_Type range Mode_In .. Mode_Linkage;

   --  Not implicit signals.
   subtype Mode_Signal_User is
     Mode_Signal_Type range Mode_Signal .. Mode_Linkage;

   --  Implicit signals.
   subtype Mode_Signal_Implicit is
     Mode_Signal_Type range Mode_Stable .. Mode_Guard;

   --  Kind of a signal.
   type Kind_Signal_Type is
     (Kind_Signal_No, Kind_Signal_Register, Kind_Signal_Bus);
   pragma Convention (C, Kind_Signal_Type);
end Grt.Types;
