with Ada.Unchecked_Deallocation;
with Ada.Unchecked_Conversion;
with Ada.Text_IO;
with Name_Table;
with Files_Map;
with Errorout; use Errorout;

package body Iirs is

   function Get_Field1 (N : Iir) return Iir;
   pragma Inline (Get_Field1);

   procedure Set_Field1 (N : Iir; V : Iir);
   pragma Inline (Set_Field1);


   function Get_Field2 (N : Iir) return Iir;
   pragma Inline (Get_Field2);

   procedure Set_Field2 (N : Iir; V : Iir);
   pragma Inline (Set_Field2);


   function Get_Field5 (N : Iir) return Iir;
   pragma Inline (Get_Field5);

   procedure Set_Field5 (N : Iir; V : Iir);
   pragma Inline (Set_Field5);


   function Get_Flag1 (N : Iir) return Boolean;
   pragma Inline (Get_Flag1);

   procedure Set_Flag1 (N : Iir; V : Boolean);
   pragma Inline (Set_Flag1);


   function Get_Flag5 (N : Iir) return Boolean;
   pragma Inline (Get_Flag5);

   procedure Set_Flag5 (N : Iir; V : Boolean);
   pragma Inline (Set_Flag5);

   --  Initialization and finalization of lists.
   procedure List_Init (List : in out Iir_List);
   procedure List_Fini (List : in out Iir_List);

   ---------------------------------------------------
   -- General subprograms that operate on every iir --
   ---------------------------------------------------

   -- This is the procedure to call when an internal consistancy test has
   -- failed.
   -- The main idea is the consistancy test *MUST* have no side effect,
   -- except calling this procedure.  To speed up, this procedure could
   -- be a no-op.
   procedure Failed (Func: String := ""; Node : Iir := Null_Iir)
   is
   begin
      if Func /= "" then
         Error_Kind (Func, Node);
      end if;
      raise Errorout.Internal_Error;
   end Failed;

   procedure Error_Kind (Msg: String; An_Iir: Iir) is
   begin
      Ada.Text_IO.Put_Line (Msg & ": can't handle "
                            & Iir_Kind'Image (Get_Kind (An_Iir))
                            & " (" & Disp_Location (An_Iir) & ')');
      raise Internal_Error;
   end Error_Kind;

   procedure Error_Kind (Msg: String; Def : Iir_Predefined_Functions) is
   begin
      Ada.Text_IO.Put_Line (Msg & ": can't handle "
                            & Iir_Predefined_Functions'Image (Def));
      raise Internal_Error;
   end Error_Kind;

   function Get_Back_End_Info (Target: Iir) return System.Address is
   begin
      return Target.Back_End_Info;
   end Get_Back_End_Info;

   procedure Set_Back_End_Info (Target: Iir; Addr: System.Address) is
   begin
      Target.Back_End_Info := Addr;
   end Set_Back_End_Info;

   --  Conversions
   --  FIXME: these conversions are unsafe!
   function To_Iir is new Ada.Unchecked_Conversion
     (Source => Integer, Target => Iir);
   function To_Integer is new Ada.Unchecked_Conversion
     (Source => Iir, Target => Integer);

   function Get_Field7 (Node: Iir) return Iir;
   pragma Inline (Get_Field7);
   procedure Set_Field7 (Node: Iir; Val: Iir);
   pragma Inline (Set_Field7);
   function Get_Field6 (Node: Iir) return Iir;
   pragma Inline (Get_Field6);
   procedure Set_Field6 (Node: Iir; Val: Iir);
   pragma Inline (Set_Field6);

   function Get_Field1 (N : Iir) return Iir is
   begin
      return N.Field1;
   end Get_Field1;

   procedure Set_Field1 (N : Iir; V : Iir) is
   begin
      N.Field1 := V;
   end Set_Field1;

   function Get_Field2 (N : Iir) return Iir is
   begin
      return N.Field2;
   end Get_Field2;

   procedure Set_Field2 (N : Iir; V : Iir) is
   begin
      N.Field2 := V;
   end Set_Field2;

   function Get_Field5 (N : Iir) return Iir is
   begin
      return N.Field5;
   end Get_Field5;

   procedure Set_Field5 (N : Iir; V : Iir) is
   begin
      N.Field5 := V;
   end Set_Field5;

   function Get_Field6 (Node: Iir) return Iir is
   begin
      return To_Iir (Node.Nbr2);
   end Get_Field6;

   procedure Set_Field6 (Node: Iir; Val: Iir) is
   begin
      Node.Nbr2 := To_Integer (Val);
   end Set_Field6;

   function Get_Field7 (Node: Iir) return Iir is
   begin
      return To_Iir (Node.Nbr3);
   end Get_Field7;

   procedure Set_Field7 (Node: Iir; Val: Iir) is
   begin
      Node.Nbr3 := To_Integer (Val);
   end Set_Field7;

   function Get_Flag1 (N : Iir) return Boolean is
   begin
      return N.Flag1;
   end Get_Flag1;

   procedure Set_Flag1 (N : Iir; V : Boolean) is
   begin
      N.Flag1 := V;
   end Set_Flag1;

   function Get_Flag5 (N : Iir) return Boolean is
   begin
      return N.Flag5;
   end Get_Flag5;

   procedure Set_Flag5 (N : Iir; V : Boolean) is
   begin
      N.Flag5 := V;
   end Set_Flag5;

   function Create_Iir_Error return Iir is
   begin
      return new Iir_Node (Iir_Kind_Error);
   end Create_Iir_Error;

   function Create_Iir (Kind: Iir_Kind) return Iir is
      Res: Iir;
   begin
      Res := new Iir_Node (Kind);
      case Kind is
         when Iir_Kind_Error =>
            raise Internal_Error;

         when Iir_Kind_Design_File =>
            Res.Field1 := new Iir_Node (Iir_Kind_Design_Unit_List);
         when Iir_Kind_Design_Unit =>
            Res.Field3 := new Iir_Node (Iir_Kind_Design_Unit_List);

         when Iir_Kind_Use_Clause =>
            null;
         when Iir_Kind_Library_Clause =>
            null;

         when Iir_Kind_Proxy =>
            null;
         when Iir_Kind_Association_Element_By_Expression
           | Iir_Kind_Association_Element_Open =>
            null;
         when Iir_Kind_Association_Element_By_Individual =>
            Res.Field4 := new Iir_Node (Iir_Kind_Choice_List);
         when Iir_Kind_Waveform_Element =>
            null;
         when Iir_Kind_Association_Choice_By_Expression =>
            null;
         when Iir_Kind_Association_Choice_By_Range =>
            null;
         when Iir_Kind_Association_Choice_By_Name =>
            null;
         when Iir_Kind_Association_Choice_By_Others =>
            null;
         when Iir_Kind_Association_Choice_By_None =>
            null;
         when Iir_Kind_Association_Choice_By_List =>
            Res.Field2 := new Iir_Node (Iir_Kind_Choice_List);
         when Iir_Kind_Designator_By_All =>
            null;
         when Iir_Kind_Designator_By_Others =>
            null;
         when Iir_Kind_Entity_Aspect_Entity =>
            null;
         when Iir_Kind_Entity_Aspect_Configuration =>
            null;
         when Iir_Kind_Entity_Aspect_Open =>
            null;
         when Iir_Kind_Block_Configuration =>
            --Res.Field1 := new Iir (Iir_Kind_Declaration_List);
            Res.Field2 := new Iir_Node (Iir_Kind_Configuration_Item_List);
         when Iir_Kind_Block_Header =>
            null;
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when Iir_Kind_Binding_Indication =>
            null;
         when Iir_Kind_Entity_Class =>
            null;
         when Iir_Kind_Attribute_Specification =>
            Res.Attr := new Iir_Node (Iir_Kind_Attribute_Value_List);
         when Iir_Kind_Attribute_Value =>
            null;
         when Iir_Kind_Selected_Element =>
            null;
         when Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference =>
            null;
         when Iir_Kind_Aggregate_Info =>
            null;
         when Iir_Kind_Disconnection_Specification =>
            null;

         when Iir_Kind_String_Literal =>
            null;
         when Iir_Kind_Character_Literal =>
            null;
         when Iir_Kind_Integer_Literal =>
            null;
         when Iir_Kind_Floating_Point_Literal =>
            null;
         when Iir_Kind_Null_Literal =>
            null;
         when Iir_Kind_Physical_Int_Literal
            | Iir_Kind_Physical_Fp_Literal =>
            null;
         when Iir_Kind_Bit_String_Literal =>
            null;
         when Iir_Kind_Simple_Aggregate =>
            null;

         when Iir_Kinds_List =>
            List_Init (Res);

         when Iir_Kind_Signal_Interface_Declaration =>
            null;
         when Iir_Kind_Constant_Interface_Declaration =>
            null;
         when Iir_Kind_Variable_Interface_Declaration =>
            null;
         when Iir_Kind_File_Interface_Declaration =>
            null;
         when Iir_Kind_Type_Declaration =>
            Res.Field2 := Create_Iir (Iir_Kind_Implicit_Functions_List);
         when Iir_Kind_Subtype_Declaration =>
            null;
         when Iir_Kind_Enumeration_Literal =>
            null;
         when Iir_Kind_Physical_Unit =>
            null;
         when Iir_Kind_Entity_Declaration =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Architecture_Declaration =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
            Res.Nbr2 := 0;
         when Iir_Kind_Configuration_Declaration =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Package_Declaration =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Package_Body =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Library_Declaration =>
            Res.Field1 := new Iir_Node (Iir_Kind_Design_File_List);
         when Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body =>
            Res.Field4 := new Iir_Node (Iir_Kind_Interface_List);
         when Iir_Kind_Implicit_Function_Declaration =>
            null;
         when Iir_Kind_Function_Body =>
            null;
         when Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Procedure_Declaration =>
            Res.Field4 := new Iir_Node (Iir_Kind_Interface_List);
         when Iir_Kind_Procedure_Body =>
            null;
         when Iir_Kind_Signal_Declaration =>
            null;
         when Iir_Kind_Guard_Signal_Declaration =>
            null;
         when Iir_Kind_Constant_Declaration =>
            null;
         when Iir_Kind_Iterator_Declaration =>
            null;
         when Iir_Kind_Variable_Declaration =>
            null;
         when Iir_Kind_File_Declaration =>
            null;
         when Iir_Kind_Alias_Declaration =>
            null;
         when Iir_Kind_Component_Declaration =>
            null;
         when Iir_Kind_Element_Declaration =>
            null;
         when Iir_Kind_Attribute_Declaration =>
            null;
         when Iir_Kind_Implicit_Signal_Declaration =>
            null;
         when Iir_Kind_Group_Template_Declaration =>
            Res.Field1 := new Iir_Node (Iir_Kind_Entity_Class_Entry_List);
         when Iir_Kind_Group_Declaration =>
            Res.Field1 := new Iir_Node (Iir_Kind_Group_Constituent_List);

         when Iir_Kind_Enumeration_Type_Definition =>
            Res.Field3 := new Iir_Node (Iir_Kind_Enumeration_Literal_List);
         when Iir_Kind_Enumeration_Subtype_Definition =>
            null;
         when Iir_Kind_Range_Expression =>
            null;
         when Iir_Kind_Integer_Type_Definition =>
            null;
         when Iir_Kind_Integer_Subtype_Definition =>
            null;
         when Iir_Kind_Floating_Type_Definition =>
            null;
         when Iir_Kind_Floating_Subtype_Definition =>
            null;
         when Iir_Kind_Array_Subtype_Definition =>
            Res.Field2 := new Iir_Node (Iir_Kind_List);
         when Iir_Kind_Unconstrained_Array_Subtype_Definition =>
            null;
         when Iir_Kind_Array_Type_Definition =>
            Res.Field2 := new Iir_Node (Iir_Kind_List);
         when Iir_Kind_Record_Type_Definition =>
            Res.Field2 := new Iir_Node (Iir_Kind_Element_Declaration_List);
         when Iir_Kind_Record_Subtype_Definition =>
            null;
         when Iir_Kind_Physical_Type_Definition =>
            Res.Field3 := new Iir_Node (Iir_Kind_Unit_List);
         when Iir_Kind_Physical_Subtype_Definition =>
            null;
         when Iir_Kind_Access_Type_Definition =>
            null;
         when Iir_Kind_Access_Subtype_Definition =>
            null;
         when Iir_Kind_Incomplete_Type_Definition =>
            Res.Field2 := new Iir_Node (Iir_Kind_List);
         when Iir_Kind_File_Type_Definition =>
            null;
         when Iir_Kind_Subtype_Definition =>
            null;

         when Iir_Kind_Block_Statement =>
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Generate_Statement =>
            null;
         when Iir_Kind_Component_Instantiation_Statement =>
            null;
         when Iir_Kind_Concurrent_Conditional_Signal_Assignment =>
            null;
         when Iir_Kind_Concurrent_Selected_Signal_Assignment =>
            Res.Field1 := new Iir_Node (Iir_Kind_Association_Choices_List);
         when Iir_Kind_Sensitized_Process_Statement =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
            Res.Field4 := new Iir_Node (Iir_Kind_Designator_List);
         when Iir_Kind_Process_Statement =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
            Res.Field3 := new Iir_Node (Iir_Kind_Declaration_List);
         when Iir_Kind_Concurrent_Assertion_Statement =>
            null;
         when Iir_Kind_Concurrent_Procedure_Call_Statement =>
            null;

         when Iir_Kind_If_Statement =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
         when Iir_Kind_Elsif =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
         when Iir_Kind_For_Loop_Statement =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
         when Iir_Kind_While_Loop_Statement =>
            Res.Field2 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
         when Iir_Kind_Next_Statement
           | Iir_Kind_Exit_Statement =>
            null;
         when Iir_Kind_Signal_Assignment_Statement =>
            -- waveform list is not initialized.
            Res.Field3 := null;
         when Iir_Kind_Null_Statement =>
            null;
         when Iir_Kind_Report_Statement =>
            null;
         when Iir_Kind_Assertion_Statement =>
            null;
         when Iir_Kind_Wait_Statement =>
            null;
         when Iir_Kind_Return_Statement =>
            null;
         when Iir_Kind_Variable_Assignment_Statement =>
            null;
         when Iir_Kind_Case_Statement =>
            Res.Field4 := new Iir_Node (Iir_Kind_Association_Choices_List);
         when Iir_Kind_Procedure_Call =>
            null;

         when Iir_Kinds_Dyadic_Operator =>
            null;
         when Iir_Kinds_Monadic_Operator =>
            null;
         when Iir_Kind_Function_Call =>
            null;
         when Iir_Kind_Qualified_Expression
           | Iir_Kind_Type_Conversion =>
            null;
         when Iir_Kind_Allocator_By_Expression
           | Iir_Kind_Allocator_By_Subtype =>
            null;

         when Iir_Kind_Simple_Name =>
            null;
         when Iir_Kind_Indexed_Name =>
            Res.Field2 := new Iir_Node (Iir_Kind_Index_List);
         when Iir_Kind_Slice_Name =>
            null;
         when Iir_Kind_Selected_Name =>
            null;
         when Iir_Kind_Selected_By_All_Name =>
            null;
         when Iir_Kind_Parenthesis_Name =>
            null;
         when Iir_Kind_Aggregate =>
            Res.Field4 := new Iir_Node (Iir_Kind_Association_Choices_List);

         when Iir_Kinds_Attribute
           | Iir_Kind_Attribute =>
            null;

         when Iir_Kind_Conditional_Waveform =>
            null;
      end case;

      return Res;
   end Create_Iir;

   procedure Free is new Ada.Unchecked_Deallocation
     (Object => Iir_Node, Name => Iir);

   procedure Unchecked_Deallocation is new Ada.Unchecked_Deallocation
     (Object => Iir_Array, Name => Iir_Array_Acc);

   procedure Free_Iir (Target: in out Iir) is
   begin
      if Target = null then
         return;
      end if;
      case Get_Kind (Target) is
         when Iir_Kind_Error =>
            null;
         when Iir_Kind_Design_File =>
            Free_Iir (Target.Field1);
         when Iir_Kind_Design_Unit =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Use_Clause =>
            null;
         when Iir_Kind_Library_Clause =>
            null;

         when Iir_Kind_Proxy =>
            null;
         when Iir_Kinds_Association_Element =>
            null;
         when Iir_Kind_Waveform_Element =>
            null;
         when Iir_Kind_Association_Choice_By_Expression =>
            null;
         when Iir_Kind_Association_Choice_By_Range =>
            null;
         when Iir_Kind_Association_Choice_By_Name =>
            null;
         when Iir_Kind_Association_Choice_By_Others =>
            null;
         when Iir_Kind_Association_Choice_By_None =>
            null;
         when Iir_Kind_Association_Choice_By_List =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Designator_By_All =>
            null;
         when Iir_Kind_Designator_By_Others =>
            null;
         when Iir_Kind_Entity_Aspect_Entity =>
            null;
         when Iir_Kind_Entity_Aspect_Configuration =>
            null;
         when Iir_Kind_Entity_Aspect_Open =>
            null;
         when Iir_Kind_Block_Configuration =>
            Free_Iir (Target.Field1);
            Free_Iir (Target.Field2);
         when Iir_Kind_Block_Header =>
            null;
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when Iir_Kind_Binding_Indication =>
            null;
         when Iir_Kind_Entity_Class =>
            null;
         when Iir_Kind_Attribute_Specification =>
            null;
         when Iir_Kind_Attribute_Value =>
            null;
         when Iir_Kind_Selected_Element =>
            null;
         when Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference =>
            null;
         when Iir_Kind_Aggregate_Info =>
            null;
         when Iir_Kind_Disconnection_Specification =>
            null;

         when Iir_Kind_String_Literal =>
            null;
         when Iir_Kind_Character_Literal =>
            null;
         when Iir_Kind_Integer_Literal =>
            null;
         when Iir_Kind_Floating_Point_Literal =>
            null;
         when Iir_Kind_Null_Literal =>
            null;
         when Iir_Kind_Bit_String_Literal =>
            null;
         when Iir_Kind_Simple_Aggregate =>
            null;

         when Iir_Kinds_List =>
            List_Fini (Target);

         when Iir_Kind_Signal_Interface_Declaration =>
            null;
         when Iir_Kind_Constant_Interface_Declaration =>
            null;
         when Iir_Kind_Variable_Interface_Declaration =>
            null;
         when Iir_Kind_File_Interface_Declaration =>
            null;
         when Iir_Kind_Type_Declaration =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Subtype_Declaration =>
            null;
         when Iir_Kind_Enumeration_Literal =>
            null;
         when Iir_Kind_Physical_Unit =>
            null;
         when Iir_Kind_Entity_Declaration =>
            Free_Iir (Target.Field2);
            Free_Iir (Target.Field3);
         when Iir_Kind_Architecture_Declaration =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Configuration_Declaration =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Package_Declaration =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Package_Body =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Library_Declaration =>
            Free_Iir (Target.Field1);
         when Iir_Kind_Function_Specification
           | Iir_Kind_Procedure_Specification =>
            Free_Iir (Target.Field4);
         when Iir_Kind_Function_Body
           | Iir_Kind_Procedure_Body =>
            Free_Iir (Target.Field2);
            Free_Iir (Target.Field3);
         when Iir_Kind_Function_Spec_Body
           | Iir_Kind_Procedure_Spec_Body =>
            Free_Iir (Target.Field4);
            Free_Iir (Target.Field2);
            Free_Iir (Target.Field3);
         when Iir_Kind_Implicit_Function_Declaration =>
            Free_Iir (Target.Field4);
         when Iir_Kind_Implicit_Procedure_Declaration =>
            Free_Iir (Target.Field4);
         when Iir_Kind_Signal_Declaration =>
            null;
         when Iir_Kind_Guard_Signal_Declaration =>
            null;
         when Iir_Kind_Constant_Declaration =>
            null;
         when Iir_Kind_Iterator_Declaration =>
            null;
         when Iir_Kind_Variable_Declaration =>
            null;
         when Iir_Kind_File_Declaration =>
            null;
         when Iir_Kind_Alias_Declaration =>
            null;
         when Iir_Kind_Component_Declaration =>
            null;
         when Iir_Kind_Element_Declaration =>
            null;
         when Iir_Kind_Attribute_Declaration =>
            null;
         when Iir_Kind_Implicit_Signal_Declaration =>
            null;
         when Iir_Kind_Group_Template_Declaration =>
            Free_Iir (Target.Field1);
         when Iir_Kind_Group_Declaration =>
            Free_Iir (Target.Field1);

         when Iir_Kind_Enumeration_Type_Definition =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Enumeration_Subtype_Definition =>
            null;
         when Iir_Kind_Range_Expression =>
            null;
         when Iir_Kind_Integer_Type_Definition =>
            null;
         when Iir_Kind_Integer_Subtype_Definition =>
            null;
         when Iir_Kind_Floating_Type_Definition =>
            null;
         when Iir_Kind_Floating_Subtype_Definition =>
            null;
         when Iir_Kind_Subtype_Definition =>
            null;
         when Iir_Kind_Array_Subtype_Definition =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Unconstrained_Array_Subtype_Definition =>
            null;
         when Iir_Kind_Array_Type_Definition =>
            null;
         when Iir_Kind_Record_Type_Definition =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Record_Subtype_Definition =>
            null;
         when Iir_Kind_Physical_Type_Definition =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Physical_Subtype_Definition =>
            null;
         when Iir_Kind_Access_Type_Definition =>
            null;
         when Iir_Kind_Access_Subtype_Definition =>
            null;
         when Iir_Kind_Incomplete_Type_Definition =>
            Free_Iir (Target.Field2);
         when Iir_Kind_File_Type_Definition =>
            null;

         when Iir_Kind_Block_Statement =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Generate_Statement =>
            Free_Iir (Target.Field3);
         when Iir_Kind_Component_Instantiation_Statement =>
            null;
         when Iir_Kind_Sensitized_Process_Statement =>
            Free_Iir (Target.Field2);
            Free_Iir (Target.Field3);
            Free_Iir (Target.Field4);
         when Iir_Kind_Process_Statement =>
            Free_Iir (Target.Field2);
            Free_Iir (Target.Field3);
         when Iir_Kind_Concurrent_Assertion_Statement =>
            null;
         when Iir_Kind_Concurrent_Conditional_Signal_Assignment =>
            -- waveform list is not destroyed.
            null;
         when Iir_Kind_Concurrent_Selected_Signal_Assignment =>
            -- selected waveform list is not destroyed.
            null;
         when Iir_Kind_Concurrent_Procedure_Call_Statement =>
            null;

         when Iir_Kind_If_Statement =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Elsif =>
            Free_Iir (Target.Field2);
         when Iir_Kind_For_Loop_Statement =>
            Free_Iir (Target.Field2);
         when Iir_Kind_While_Loop_Statement =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Next_Statement
           | Iir_Kind_Exit_Statement =>
            null;
         when Iir_Kind_Signal_Assignment_Statement =>
            null;
         when Iir_Kind_Assertion_Statement =>
            null;
         when Iir_Kind_Report_Statement =>
            null;
         when Iir_Kind_Wait_Statement =>
            null;
         when Iir_Kind_Variable_Assignment_Statement =>
            null;
         when Iir_Kind_Return_Statement =>
            null;
         when Iir_Kind_Null_Statement =>
            null;
         when Iir_Kind_Case_Statement =>
            Free_Iir (Target.Field4);
         when Iir_Kind_Procedure_Call =>
            null;

         when Iir_Kinds_Dyadic_Operator =>
            null;
         when Iir_Kinds_Monadic_Operator =>
            null;
         when Iir_Kind_Function_Call =>
            null;
         when Iir_Kind_Qualified_Expression
           | Iir_Kind_Type_Conversion =>
            null;
         when Iir_Kind_Allocator_By_Expression
           | Iir_Kind_Allocator_By_Subtype =>
            null;

         when Iir_Kind_Simple_Name =>
            null;
         when Iir_Kind_Indexed_Name =>
            Free_Iir (Target.Field2);
         when Iir_Kind_Slice_Name =>
            null;
         when Iir_Kind_Selected_Name =>
            null;
         when Iir_Kind_Selected_By_All_Name =>
            null;

         when Iir_Kind_Parenthesis_Name =>
            null;
         when Iir_Kind_Aggregate =>
            Free_Iir (Target.Field4);
         when Iir_Kind_Physical_Int_Literal
            | Iir_Kind_Physical_Fp_Literal =>
            null;

         when Iir_Kinds_Attribute
           | Iir_Kind_Attribute =>
            null;

         when Iir_Kind_Conditional_Waveform =>
            Error_Kind ("Free_iir", Target);
      end case;
      Free (Target);
   end Free_Iir;

   procedure Set_Location (Target: Iir; Location: Location_Type) is
   begin
      Target.Location := Location;
   end Set_Location;

   function Get_Location (Target: Iir) return Location_Type is
   begin
      return Target.Location;
   end Get_Location;

   function Get_Column_Number (Target: Iir) return Natural is
      Line, Col: Natural;
      Name : Name_Id;
   begin
      Files_Map.Location_To_Position
        (Get_Location (Target), Name, Line, Col);
      return Col;
   end Get_Column_Number;

   function Get_Line_Number (Target: Iir) return Natural is
      Line, Col: Natural;
      Name : Name_Id;
   begin
      Files_Map.Location_To_Position
        (Get_Location (Target), Name, Line, Col);
      return Line;
   end Get_Line_Number;

   function Get_File_Name (Target: Iir) return Name_Id is
      Line, Col: Natural;
      Name : Name_Id;
   begin
      Files_Map.Location_To_Position
        (Get_Location (Target), Name, Line, Col);
      return Name;
   end Get_File_Name;

   procedure Location_Copy (Target: Iir; Src: Iir) is
   begin
      Set_Location (Target, Get_Location (Src));
   end Location_Copy;

   -- Get kind
   function Get_Kind (An_Iir: Iir) return Iir_Kind is
   begin
      return An_Iir.Kind;
   end Get_Kind;

   function Clone_Iir (Src : Iir; New_Kind : Iir_Kind) return Iir
   is
      Res : Iir;
   begin
      Res := new Iir_Node (New_Kind);
      Res.Flag1 := Src.Flag1;
      Res.Flag2 := Src.Flag2;
      Res.Flag3 := Src.Flag3;
      Res.Flag4 := Src.Flag4;
      Res.Flag5 := Src.Flag5;
      Res.Flag6 := Src.Flag6;
      Res.Flag7 := Src.Flag7;
      Res.Flag8 := Src.Flag8;
      Res.State1 := Src.State1;
      Res.State2 := Src.State2;
      Res.State3 := Src.State3;
      Res.Staticness_1 := Src.Staticness_1;
      Res.Staticness_2 := Src.Staticness_2;
      Res.Odigit_1 := Src.Odigit_1;
      Res.Odigit_2 := Src.Odigit_2;
      Res.Location := Src.Location;
      Res.Back_End_Info := Src.Back_End_Info;
      Res.Identifier := Src.Identifier;
      Res.Field1 := Src.Field1;
      Res.Field2 := Src.Field2;
      Res.Field3 := Src.Field3;
      Res.Field4 := Src.Field4;
      Res.Field5 := Src.Field5;
      Res.Nbr2 := Src.Nbr2;
      Res.Nbr3 := Src.Nbr3;

      Src.Identifier := Null_Identifier;
      Src.Field1 := null;
      Src.Field2 := null;
      Src.Field3 := null;
      Src.Field4 := null;
      Src.Field5 := null;
      return Res;
   end Clone_Iir;


   -----------------
   -- design file --
   -----------------

   -- Iir_Design_File

   function Get_Design_Unit_List (Design: Iir_Design_File)
                                  return Iir_Design_Unit_List is
   begin
      return Design.Field1;
   end Get_Design_Unit_List;

   procedure Check_Kind_For_Library_Declaration (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Library_Clause =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Library_Declaration;

   function Get_Library_Declaration (Design : Iir)
     return Iir_Library_Declaration is
   begin
      Check_Kind_For_Library_Declaration (Design);
      return Design.Field2;
   end Get_Library_Declaration;

   procedure Set_Library_Declaration (Design : Iir;
                                      Library : Iir_Library_Declaration) is
   begin
      Check_Kind_For_Library_Declaration (Design);
      Design.Field2 := Library;
   end Set_Library_Declaration;

   function Get_Design_File (Unit : Iir_Design_Unit) return Iir_Design_File
   is
   begin
      return Unit.Parent;
   end Get_Design_File;

   procedure Set_Design_File (Unit : Iir_Design_Unit; File : Iir_Design_File)
   is
   begin
      Unit.Parent := File;
   end Set_Design_File;

   function Time_Stamp_Acc_To_Iir is new Ada.Unchecked_Conversion
     (Source => Time_Stamp_Acc, Target => Iir);

   function Iir_To_Time_Stamp_Acc is new Ada.Unchecked_Conversion
     (Source => Iir, Target => Time_Stamp_Acc);

   procedure Set_File_Time_Stamp
     (Design: Iir_Design_File; Stamp: Time_Stamp_Acc)
   is
   begin
      Design.Field2 := Time_Stamp_Acc_To_Iir (Stamp);
   end Set_File_Time_Stamp;

   function Get_File_Time_Stamp (Design: Iir_Design_File)
     return Time_Stamp_Acc
   is
   begin
      return Iir_To_Time_Stamp_Acc (Design.Field2);
   end Get_File_Time_Stamp;

   procedure Set_Analysis_Time_Stamp
     (Design: Iir_Design_File; Stamp: Time_Stamp_Acc)
   is
   begin
      Design.Field3 := Time_Stamp_Acc_To_Iir (Stamp);
   end Set_Analysis_Time_Stamp;

   function Get_Analysis_Time_Stamp (Design: Iir_Design_File)
     return Time_Stamp_Acc
   is
   begin
      return Iir_To_Time_Stamp_Acc (Design.Field3);
   end Get_Analysis_Time_Stamp;

   function Get_Design_File_Filename (File : Iir_Design_File) return Name_Id
   is
   begin
      return Name_Id (File.Nbr3);
   end Get_Design_File_Filename;

   procedure Set_Design_File_Filename (File : Iir_Design_File; Name : Name_Id)
   is
   begin
      File.Nbr3 := Integer (Name);
   end Set_Design_File_Filename;

   function Get_Design_File_Directory (File : Iir_Design_File)
     return Name_Id is
   begin
      return Name_Id (File.Nbr2);
   end Get_Design_File_Directory;

   procedure Set_Design_File_Directory (File : Iir_Design_File; Dir : Name_Id)
   is
   begin
      File.Nbr2 := Integer (Dir);
   end Set_Design_File_Directory;

   function Get_Design_File_List (Library: Iir_Library_Declaration)
                                  return Iir_Design_File_List is
   begin
      return Library.Field1;
   end Get_Design_File_List;

   type Int_Access_Type is new Integer;
   for Int_Access_Type'Size use System.Word_Size; --Iir_Identifier_Acc'Size;

   --  Safe conversions.
   function Iir_To_Int_Access_Type is
      new Ada.Unchecked_Conversion (Source => Iir,
                                    Target => Int_Access_Type);
   function Int_Access_Type_To_Iir is
      new Ada.Unchecked_Conversion (Source => Int_Access_Type,
                                    Target => Iir);

   procedure Check_Kind_For_Date (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Library_Declaration
           | Iir_Kind_Design_Unit =>
            null;
         when others =>
            Failed ("date", Target);
      end case;
   end Check_Kind_For_Date;

   function Get_Date (Target: Iir) return Date_Type is
   begin
      Check_Kind_For_Date (Target);
      return Date_Type (Iir_To_Int_Access_Type (Target.Field5));
   end Get_Date;

   procedure Set_Date (Target: Iir; Date: Date_Type) is
   begin
      Check_Kind_For_Date (Target);
      Target.Field5 := Int_Access_Type_To_Iir (Int_Access_Type (Date));
   end Set_Date;

   function Get_Library_Directory (Library : Iir_Library_Declaration)
     return Name_Id is
   begin
      return Name_Id (Library.Nbr2);
   end Get_Library_Directory;

   procedure Set_Library_Directory (Library : Iir_Library_Declaration;
                                    Dir : Name_Id) is
   begin
      Library.Nbr2 := Integer (Dir);
   end Set_Library_Directory;

   function Get_Context_Items (Design_Unit: Iir_Design_Unit)
                               return Iir_Declaration_List is
   begin
      return Design_Unit.Field1;
   end Get_Context_Items;

   procedure Set_Context_Items (Design_Unit: Iir_Design_Unit;
                                Items_List : Iir_Declaration_List) is
   begin
      Design_Unit.Field1 := Items_List;
   end Set_Context_Items;

   function Get_Dependence_List (Design_Unit: Iir_Design_Unit)
                                 return Iir_Design_Unit_List is
   begin
      return Design_Unit.Field3;
   end Get_Dependence_List;

   procedure Set_Dependence_List (Design_Unit: Iir_Design_Unit;
                                  List : Iir_Design_Unit_List) is
   begin
      Design_Unit.Field3 := List;
   end Set_Dependence_List;

   function Get_Loaded_Flag (Design_Unit : Iir_Design_Unit) return Boolean is
   begin
      return Design_Unit.Flag1;
   end Get_Loaded_Flag;

   procedure Set_Loaded_Flag (Design_Unit : Iir_Design_Unit;
                              Flag : Boolean) is
   begin
      Design_Unit.Flag1 := Flag;
   end Set_Loaded_Flag;

   function Get_Library_Unit (Design_Unit: Iir_Design_Unit) return Iir is
   begin
      return Design_Unit.Field2;
   end Get_Library_Unit;

   procedure Set_Library_Unit (Design_Unit: Iir_Design_Unit;
                               Library_Unit: Iir) is
   begin
      Design_Unit.Field2 := Library_Unit;
   end Set_Library_Unit;

   procedure Set_Pos_Line_Off (Design_Unit: Iir_Design_Unit;
                               Pos : Source_Ptr; Line, Off: Natural) is
   begin
      Design_Unit.Field1 := To_Iir (Natural (Pos));
      Design_Unit.Nbr2 := Off;
      Design_Unit.Nbr3 := Line;
   end Set_Pos_Line_Off;

   procedure Get_Pos_Line_Off (Design_Unit: Iir_Design_Unit;
                               Pos : out Source_Ptr; Line, Off: out Natural) is
   begin
      Pos := Source_Ptr (To_Integer (Design_Unit.Field1));
      Off := Design_Unit.Nbr2;
      Line := Design_Unit.Nbr3;
   end Get_Pos_Line_Off;

   --------------
   -- Literals --
   --------------

   function Create_String (Id: Name_Id) return Iir_String_Literal is
      Res: Iir_String_Literal;
   begin
      Res := Create_Iir (Iir_Kind_String_Literal);
      Res.Nbr2 := Natural (Id);
      return Res;
   end Create_String;

   -- Create an identifier whose name is uniq.
   procedure Create_Identifier (Identifier: out Name_Id) is
      Id: Name_Id;
   begin
      Name_Table.Create_Uniq_Identifier (Id);
      Identifier := Id;
   end Create_Identifier;

   function Get_String (Id: Iir) return String is
   begin
      case Get_Kind (Id) is
         when Iir_Kind_Character_Literal =>
            return ''' & Name_Table.Get_Character (Id.Identifier) & ''';
         when Iir_Kind_String_Literal =>
            return Name_Table.Image (Id.Identifier);
         when others =>
            Error_Kind ("get_string", Id);
            return "";
      end case;
   end Get_String;

   function Get_String_Length (Id: Iir) return Natural is
   begin
      case Get_Kind (Id) is
         when Iir_Kind_Character_Literal =>
            return 3;
         when Iir_Kind_String_Literal =>
            return Name_Table.Get_Name_Length (Id.Identifier);
         when others =>
            Error_Kind ("get_string_length", Id);
            return 0;
      end case;
   end Get_String_Length;

   function Get_Text (Id: Iir_Character_Literal) return Character is
   begin
      return Name_Table.Get_Character (Name_Id (Id.Nbr2));
   end Get_Text;

   function Get_Character (Char: Name_Id) return Iir_Character_Literal is
      Res: Iir_Character_Literal;
   begin
      Res := Create_Iir (Iir_Kind_Character_Literal);
      Res.Nbr2 := Natural (Char);
      return Res;
   end Get_Character;

   function Is_Character_Equal (Left, Right: Iir_Character_Literal)
                                return Boolean is
   begin
      return Get_Text (Left) = Get_Text (Right);
   end Is_Character_Equal;

   procedure Check_Kind_For_Literal_Origin (Lit : Iir) is
   begin
      case Get_Kind (Lit) is
         when Iir_Kind_Character_Literal
           | Iir_Kind_String_Literal
           | Iir_Kind_Bit_String_Literal
           | Iir_Kind_Integer_Literal
           | Iir_Kind_Floating_Point_Literal
           | Iir_Kind_Physical_Int_Literal
           | Iir_Kind_Physical_Fp_Literal
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Simple_Aggregate =>
            null;
         when others =>
            Failed ("literal_origin", Lit);
      end case;
   end Check_Kind_For_Literal_Origin;

   function Get_Literal_Origin (Lit: Iir) return Iir is
   begin
      Check_Kind_For_Literal_Origin (Lit);
      return Lit.Field3;
   end Get_Literal_Origin;

   procedure Set_Literal_Origin (Lit: Iir; Orig : Iir) is
   begin
      Check_Kind_For_Literal_Origin (Lit);
      Lit.Field3 := Orig;
   end Set_Literal_Origin;

   -- iir_integer_literal

   procedure Check_Kind_For_Value (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Integer_Literal
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Physical_Int_Literal
           | Iir_Kind_Physical_Unit =>
            null;
         when others =>
            Failed ("Value", Target);
      end case;
   end Check_Kind_For_Value;

   function Get_Value (Lit: Iir) return Iir_Int64 is
   begin
      Check_Kind_For_Value (Lit);
      return Lit.Nbr3_Int64;
   end Get_Value;

   procedure Set_Value (Lit: Iir; Val: Iir_Int64) is
   begin
      Check_Kind_For_Value (Lit);
      Lit.Nbr3_Int64 := Val;
   end Set_Value;

   procedure Check_Kind_For_Fp_Value (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Floating_Point_Literal
           | Iir_Kind_Physical_Fp_Literal =>
            null;
         when others =>
            Error_Kind ("fp_value", Target);
      end case;
   end Check_Kind_For_Fp_Value;

   function Get_Fp_Value (Lit: Iir) return Iir_Fp64 is
   begin
      Check_Kind_For_Fp_Value (Lit);
      return Lit.Nbr3_Fp64;
   end Get_Fp_Value;

   procedure Set_Fp_Value (Lit: Iir; Val: Iir_Fp64) is
   begin
      Check_Kind_For_Fp_Value (Lit);
      Lit.Nbr3_Fp64 := Val;
   end Set_Fp_Value;

   function Get_Enumeration_Decl (Target : Iir_Enumeration_Literal)
                                 return Iir_Enumeration_Literal is
   begin
      return Get_Field2 (Target);
   end Get_Enumeration_Decl;

   procedure Set_Enumeration_Decl (Target : Iir_Enumeration_Literal;
                                   Lit : Iir_Enumeration_Literal) is
   begin
      Set_Field2 (Target, Lit);
   end Set_Enumeration_Decl;

   function Get_Simple_Aggregate_List (Target : Iir_Simple_Aggregate)
                                      return Iir_List is
   begin
      return Get_Field2 (Target);
   end Get_Simple_Aggregate_List;

   procedure Set_Simple_Aggregate_List (Target: Iir_Simple_Aggregate;
                                        List : Iir_List) is
   begin
      Set_Field2 (Target, List);
   end Set_Simple_Aggregate_List;

   function Get_Bit_String_Base (Lit : Iir_Bit_String_Literal)
     return Natural is
   begin
      return Lit.Nbr2;
   end Get_Bit_String_Base;

   procedure Set_Bit_String_Base (Lit : Iir_Bit_String_Literal; Base : Natural)
   is
   begin
      Lit.Nbr2 := Base;
   end Set_Bit_String_Base;

   function Get_Bit_String_0 (Lit : Iir_Bit_String_Literal)
     return Iir_Enumeration_Literal is
   begin
      return Lit.Field4;
   end Get_Bit_String_0;

   procedure Set_Bit_String_0 (Lit : Iir_Bit_String_Literal;
                               El : Iir_Enumeration_Literal) is
   begin
      Lit.Field4 := El;
   end Set_Bit_String_0;

   function Get_Bit_String_1 (Lit : Iir_Bit_String_Literal)
     return Iir_Enumeration_Literal is
   begin
      return Lit.Field5;
   end Get_Bit_String_1;

   procedure Set_Bit_String_1 (Lit : Iir_Bit_String_Literal;
                               El : Iir_Enumeration_Literal) is
   begin
      Lit.Field5 := El;
   end Set_Bit_String_1;

   ------------
   -- Tuples --
   ------------

   -- IIr_waveform_element methods.
   function Get_We_Value (We: Iir_Waveform_Element) return Iir is
   begin
      return We.Field1;
   end Get_We_Value;

   procedure Set_We_Value (We: Iir_Waveform_Element; An_Iir: Iir) is
   begin
      We.field1 := An_Iir;
   end Set_We_Value;

   function Get_Time (We: Iir_Waveform_Element) return Iir is
   begin
      return We.Field2;
   end Get_Time;

   procedure Set_Time (We: Iir_Waveform_Element; An_Iir: Iir) is
   begin
      We.Field2 := An_Iir;
   end Set_Time;

   procedure Check_Kind_For_Associated (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Association_By_Choice =>
            null;
         when others =>
            Failed ("associated", Target);
      end case;
   end Check_Kind_For_Associated;

   procedure Set_Associated (Target: Iir; Associated: Iir) is
   begin
      Check_Kind_For_Associated (Target);
      Target.Field1 := Associated;
   end Set_Associated;

   function Get_Associated (Target: Iir) return Iir is
   begin
      Check_Kind_For_Associated (Target);
      return Target.Field1;
   end Get_Associated;

   function Get_Choice_List (Target: Iir) return Iir_Choice_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Association_Choice_By_List =>
            null;
         when others =>
            Failed ("get_choice_list", Target);
      end case;
      return Target.Field2;
   end Get_Choice_List;

   procedure Check_Kind_For_Choice_Staticness (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Association_Choice_By_Range
           | Iir_Kind_Association_Choice_By_Expression
           | Iir_Kind_Association_Choice_By_List =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Choice_Staticness;

   function Get_Choice_Staticness (Target : Iir) return Iir_Staticness is
   begin
      Check_Kind_For_Choice_Staticness (Target);
      return Target.Staticness_2;
   end Get_Choice_Staticness;

   procedure Set_Choice_Staticness (Target: Iir; Staticness : Iir_Staticness)
   is
   begin
      Check_Kind_For_Choice_Staticness (Target);
      Target.Staticness_2 := Staticness;
   end Set_Choice_Staticness;

   procedure Set_Block_Specification (Target: Iir; Block: Iir) is
   begin
      Target.Field1 := Block;
   end Set_Block_Specification;

   function Get_Block_Specification (Target: Iir) return Iir is
   begin
      return Target.Field1;
   end Get_Block_Specification;

   procedure Set_Prev_Block_Configuration
     (Target : Iir_Block_Configuration; Block : Iir_Block_Configuration)
   is
   begin
      Target.Field4 := Block;
   end Set_Prev_Block_Configuration;

   function Get_Prev_Block_Configuration (Target : Iir_Block_Configuration)
     return Iir_Block_Configuration
   is
   begin
      return Target.Field4;
   end Get_Prev_Block_Configuration;

   function Get_Configuration_Item_List (Target: Iir)
     return Iir_Configuration_Item_List is
   begin
      return Target.Field2;
   end Get_Configuration_Item_List;

   -----------
   -- Lists --
   -----------
   --  Layout of lists:
   --  A list is stored into an IIR.
   --  There are two bounds for a list:
   --    the current number of elements
   --    the maximum number of elements.
   --  Using a maximum number of element bound (which can be increased) avoid
   --  to reallocating memory at each insertion.

   function Get_Max_Nbr_Elements (List : Iir_List) return Natural;
   pragma Inline (Get_Max_Nbr_Elements);

   procedure Set_Max_Nbr_Elements (List : Iir_List; Max : Natural);
   pragma Inline (Set_Max_Nbr_Elements);

   function List_Is_Self_Contained (List : Iir_List) return Boolean;
   pragma Inline (List_Is_Self_Contained);

   procedure List_Set_Nbr_Elements (List : Iir_List; Nbr : Natural);
   pragma Inline (List_Set_Nbr_Elements);

   function Get_Nbr_Elements (List: Iir_List) return Natural is
   begin
      return List.Nbr2;
   end Get_Nbr_Elements;

   procedure List_Set_Nbr_Elements (List : Iir_List; Nbr : Natural) is
   begin
      List.Nbr2 := Nbr;
   end List_Set_Nbr_Elements;

   function Get_Max_Nbr_Elements (List : Iir_List) return Natural is
   begin
      return List.Nbr3;
   end Get_Max_Nbr_Elements;

   procedure Set_Max_Nbr_Elements (List : Iir_List; Max : Natural) is
   begin
      List.Nbr3 := Max;
   end Set_Max_Nbr_Elements;

   function List_Is_Self_Contained (List : Iir_List) return Boolean is
   begin
      return Get_Max_Nbr_Elements (List) <= 5;
   end List_Is_Self_Contained;

   type Iir_Array_Fat is new Iir_Array (Natural);
   type Iir_Array_Fat_Acc is access Iir_Array_Fat;

   --  Safe conversions.
   function Iir_To_Iir_Array_Fat_Acc is
      new Ada.Unchecked_Conversion (Source => Iir,
                                    Target => Iir_Array_Fat_Acc);
   function Iir_Array_Fat_Acc_To_Iir is
      new Ada.Unchecked_Conversion (Source => Iir_Array_Fat_Acc,
                                    Target => Iir);

   procedure List_Init (List : in out Iir_List) is
   begin
      Set_Max_Nbr_Elements (List, 5);
      List_Set_Nbr_Elements (List, 0);
   end List_Init;

   procedure List_Fini (List : in out Iir_List) is
      procedure Free (Ptr : Iir_Array_Fat_Acc);
      pragma Import (C, Free, "free");
   begin
      if not List_Is_Self_Contained (List) then
         Free (Iir_To_Iir_Array_Fat_Acc (List.Field1));
      end if;
   end List_Fini;

   function Get_Nth_Element (List: Iir_List; N: Natural)
     return Iir
   is
   begin
      if Get_Kind (List) not in Iir_Kinds_List then
         Error_Kind ("get_nth_element", List);
      end if;
      if N >= Get_Nbr_Elements (List) then
         return null;
      end if;
      if List_Is_Self_Contained (List) then
         case N is
            when 0 =>
               return List.Field1;
            when 1 =>
               return List.Field2;
            when 2 =>
               return List.Field3;
            when 3 =>
               return List.Field4;
            when 4 =>
               return List.Field5;
            when others =>
               raise Internal_Error;
         end case;
      else
         return Iir_To_Iir_Array_Fat_Acc (List.Field1) (N);
      end if;
   end Get_Nth_Element;

   -- Replace an element selected by position.
   procedure Replace_Nth_Element (List: Iir_List; N: Natural; El: Iir) is
   begin
      if N >= Get_Nbr_Elements (List) then
         raise Internal_Error;
      end if;
      if List_Is_Self_Contained (List) then
         case N is
            when 0 =>
               List.Field1 := El;
            when 1 =>
               List.Field2 := El;
            when 2 =>
               List.Field3 := El;
            when 3 =>
               List.Field4 := El;
            when 4 =>
               List.Field5 := El;
            when others =>
               raise Internal_Error;
         end case;
      else
         Iir_To_Iir_Array_Fat_Acc (List.Field1) (N) := El;
      end if;
   end Replace_Nth_Element;

   -- Be sure an element can be added to LIST.
   -- It doesn't change the number of elements.
   procedure List_Grow (List: Iir) is
      --  Be careful: size in bytes.
      function Alloc (Size : Natural) return Iir_Array_Fat_Acc;
      pragma Import (C, Alloc, "malloc");

      function Realloc (Ptr : Iir_Array_Fat_Acc; Size : Natural)
        return Iir_Array_Fat_Acc;
      pragma Import (C, Realloc, "realloc");

      Tmp : Iir_Array_Fat_Acc;
      N : Natural;
   begin
      if Get_Nbr_Elements (List) < Get_Max_Nbr_Elements (List) then
         return;
      end if;
      if List_Is_Self_Contained (List) then
         N := 10;
         Tmp := Alloc (N * Iir'Size / System.Storage_Unit);
         Tmp (0) := List.Field1;
         Tmp (1) := List.Field2;
         Tmp (2) := List.Field3;
         Tmp (3) := List.Field4;
         Tmp (4) := List.Field5;
      else
         N := Get_Max_Nbr_Elements (List) + 10;
         Tmp := Realloc (Iir_To_Iir_Array_Fat_Acc (List.Field1),
                         N * Iir'Size / System.Storage_Unit);
      end if;
      List.Field1 := Iir_Array_Fat_Acc_To_Iir (Tmp);
      Set_Max_Nbr_Elements (List, N);
   end List_Grow;

   procedure Append_Element (List: Iir_List; Element: Iir)
   is
      N : Natural := Get_Nbr_Elements (List);
   begin
      List_Grow (List);

      List_Set_Nbr_Elements (List, N + 1);
      Replace_Nth_Element (List, N, Element);
   end Append_Element;

   -- Return the last element of the list, or null.
   function Get_Last_Element (List: Iir_List) return Iir
   is
      N : Natural := Get_Nbr_Elements (List);
   begin
      if N = 0 then
         return null;
      else
         return Get_Nth_Element (List, N - 1);
      end if;
   end Get_Last_Element;

   -- Return the first element of the list, or null.
   function Get_First_Element (List: Iir_List) return Iir is
   begin
      return Get_Nth_Element (List, 0);
   end Get_First_Element;

   -- Add (append) an element only if it was not already present in the list.
   procedure Add_Element (List: Iir_List; El: Iir)
   is
      Nbr : Natural := Get_Nbr_Elements (List);
   begin
      for I in 0 .. Nbr - 1 loop
         if Get_Nth_Element (List, I) = El then
            return;
         end if;
      end loop;

      Append_Element (List, El);
   end Add_Element;

   procedure Remove_Nth_Element (List: Iir_List; N: Natural)
   is
      Nbr : Natural := Get_Nbr_Elements (List);
   begin
      if N >= Nbr then
         raise Internal_Error;
      end if;
      for I in N .. Nbr - 2 loop
         Replace_Nth_Element (List, I, Get_Nth_Element (List, I + 1));
      end loop;
      Set_Nbr_Elements (List, Nbr - 1);
   end Remove_Nth_Element;

   procedure Set_Nbr_Elements (List: Iir_List; N: Natural) is
   begin
      if N > Get_Nbr_Elements (List) then
         raise Internal_Error;
      end if;
      List_Set_Nbr_Elements (List, N);
   end Set_Nbr_Elements;

   -- Return the position of the last element.
   -- Return -1 if the list is empty.
   function Get_Last_Element_Position (List: Iir_List) return Integer is
   begin
      return Get_Nbr_Elements (List) - 1;
   end Get_Last_Element_Position;

   function Get_Nbr_Elements_Safe (List: Iir_List) return Natural is
   begin
      if List = Null_Iir then
         return 0;
      else
         return Get_Nbr_Elements (List);
      end if;
   end Get_Nbr_Elements_Safe;

   -- Empty the list
   procedure Empty_List (List: Iir_List) is
   begin
      Set_Nbr_Elements (List, 0);
   end Empty_List;

   ------------------
   -- Declarations --
   ------------------

   -- For iir_architecture_declaration.
   procedure Set_Entity (Target: Iir; Entity: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Architecture_Declaration
           | Iir_Kind_Configuration_Declaration
           | Iir_Kind_Entity_Aspect_Entity =>
            null;
         when others =>
            Failed;
      end case;

      Target.Field1 := Entity;
   end Set_Entity;

   function Get_Entity (Decl: Iir) return Iir is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Architecture_Declaration
           | Iir_Kind_Configuration_Declaration
           | Iir_Kind_Entity_Aspect_Entity =>
            null;
         when others =>
            Failed;
      end case;

      return Decl.Field1;
   end Get_Entity;

   procedure Set_Architecture (Target: Iir_Entity_Aspect_Entity; Arch : Iir) is
   begin
      Target.Field2 := Arch;
   end Set_Architecture;

   function Get_Architecture (Target: Iir_Entity_Aspect_Entity)
                              return Iir is
   begin
      return Target.Field2;
   end Get_Architecture;

   procedure Set_Default_Configuration_Declaration
     (Target : Iir_Architecture_Declaration; Conf : Iir)
   is
   begin
      Target.Field4 := Conf;
   end Set_Default_Configuration_Declaration;

   function Get_Default_Configuration_Declaration
     (Target : Iir_Architecture_Declaration)
     return Iir
   is
   begin
      return Target.Field4;
   end Get_Default_Configuration_Declaration;

   procedure Check_Kind_For_Package_Body (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Package_Body =>
            null;
         when others =>
            Failed ("check_kind_for_package_body", Target);
      end case;
   end Check_Kind_For_Package_Body;

   procedure Set_Package (Package_Body: Iir; Decl: Iir_Package_Declaration) is
   begin
      Check_Kind_For_Package_Body (Package_Body);
      Package_Body.Field1 := Decl;
   end Set_Package;

   function Get_Package (Package_Body: Iir) return Iir_Package_Declaration is
   begin
      Check_Kind_For_Package_Body (Package_Body);
      return Package_Body.Field1;
   end Get_Package;

   procedure Set_Need_Body (Decl : Iir_Package_Declaration; Flag : Boolean) is
   begin
      Decl.Flag1 := Flag;
   end Set_Need_Body;

   function Get_Need_Body (Decl : Iir_Package_Declaration) return Boolean is
   begin
      return Decl.Flag1;
   end Get_Need_Body;

   procedure Set_Block_Configuration
     (Target: Iir; Block: Iir_Block_Configuration) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Configuration_Declaration
           | Iir_Kind_Component_Configuration =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field2 := Block;
   end Set_Block_Configuration;

   function Get_Block_Configuration (Target: Iir)
                                     return Iir_Block_Configuration is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Configuration_Declaration
           | Iir_Kind_Component_Configuration =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Block_Configuration;

   -- iir_entity_declaration:
   -- Field1: used by port_list.
   procedure Set_Port_List (Target: Iir; Ports: Iir_Port_List) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Declaration
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Block_Header =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field1 := Ports;
   end Set_Port_List;

   function Get_Port_List (Target: Iir) return Iir_Port_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Declaration
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Block_Header =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field1;
   end Get_Port_List;

   procedure Set_Generic_List (Target: Iir; Generics: Iir_Generic_List) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Declaration
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Block_Header =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field4 := Generics;
   end Set_Generic_List;

   function Get_Generic_List (Target: Iir) return Iir_Generic_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Declaration
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Block_Header =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field4;
   end Get_Generic_List;

   procedure Set_Binding_Indication (Target : Iir; Binding : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field3 := Binding;
   end Set_Binding_Indication;

   function Get_Binding_Indication (Target : Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field3;
   end Get_Binding_Indication;

   procedure Check_Kind_For_Type (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Type_Declaration
           | Iir_Kind_Subtype_Declaration
           | Iir_Kind_Range_Expression
           | Iir_Kinds_Attribute
           | Iir_Kind_Attribute_Value
           | Iir_Kind_Attribute
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Character_Literal
           | Iir_Kind_Integer_Literal
           | Iir_Kind_Floating_Point_Literal
           | Iir_Kind_Null_Literal
           | Iir_Kind_String_Literal
           | Iir_Kind_Bit_String_Literal
           | Iir_Kind_Physical_Fp_Literal
           | Iir_Kind_Physical_Int_Literal
           | Iir_Kind_Physical_Unit
           | Iir_Kind_Simple_Aggregate
           | Iir_Kinds_Dyadic_Operator
           | Iir_Kinds_Monadic_Operator
           | Iir_Kind_Qualified_Expression
           | Iir_Kind_Type_Conversion
           | Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Allocator_By_Expression
           | Iir_Kind_Allocator_By_Subtype
           | Iir_Kinds_Object_Declaration
           | Iir_Kinds_Name
           | Iir_Kind_Element_Declaration
           | Iir_Kind_Attribute_Declaration
           | Iir_Kind_Function_Call
           | Iir_Kind_Return_Statement
           | Iir_Kind_Aggregate
           | Iir_Kind_Selected_Element
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference
           | Iir_Kind_Disconnection_Specification =>
            null;
         when others =>
            Failed ("get/set_type", Target);
      end case;
   end Check_Kind_For_Type;

   -- iir_type_declaration
   procedure Set_Type (Target: Iir; Atype: Iir) is
   begin
      Check_Kind_For_Type (Target);
      Target.Field1 := Atype;
   end Set_Type;

   function Get_Type (Target: Iir)
     return Iir is
   begin
      Check_Kind_For_Type (Target);
      return Target.Field1;
   end Get_Type;

   procedure Check_Kind_For_Base_Name (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Alias_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_File_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration
           | Iir_Kind_Slice_Name
           | Iir_Kind_Indexed_Name
           | Iir_Kind_Selected_Element
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference
           | Iir_Kind_Selected_By_All_Name
           | Iir_Kind_Attribute_Value
           | Iir_Kind_Function_Call
           | Iir_Kinds_Signal_Attribute =>
            null;
         when others =>
            Failed ("base_name", Target);
      end case;
   end Check_Kind_For_Base_Name;

   procedure Set_Base_Name (Target : Iir; Name : Iir) is
   begin
      Check_Kind_For_Base_Name (Target);
      Target.Field4 := Name;
   end Set_Base_Name;

   function Get_Base_Name (Target : Iir) return Iir is
   begin
      Check_Kind_For_Base_Name (Target);
      return Target.Field4;
   end Get_Base_Name;

   function Get_Implicit_Functions_List (Decl : Iir_Type_Declaration)
                                         return Iir_Implicit_Functions_List
   is
   begin
      return Decl.Field2;
   end Get_Implicit_Functions_List;

   function Get_Anonymous_Interface (Decl : Iir_Type_Declaration)
                                     return Iir is
   begin
      return Decl.Field3;
   end Get_Anonymous_Interface;

   procedure Set_Anonymous_Interface (Decl : Iir_Type_Declaration;
                                      Interface : Iir) is
   begin
      Decl.Field3 := Interface;
   end Set_Anonymous_Interface;

   procedure Check_Kind_For_Signal (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Signal;

   procedure Set_Signal_Kind (Target: Iir; Signal_Kind: Iir_Signal_Kind) is
   begin
      Check_Kind_For_Signal (Target);
      Target.State1 := Iir_Signal_Kind'Pos (Signal_Kind);
   end Set_Signal_Kind;

   function Get_Signal_Kind (Target: Iir) return Iir_Signal_Kind is
   begin
      Check_Kind_For_Signal (Target);
      return Iir_Signal_Kind'Val (Target.State1);
   end Get_Signal_Kind;

   procedure Check_Kind_For_Implicit_Signal_Chain (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Implicit_Signal_Declaration
           | Iir_Kind_Stable_Attribute
           | Iir_Kind_Quiet_Attribute
           | Iir_Kind_Delayed_Attribute
           | Iir_Kind_Transaction_Attribute =>
            null;
         when others =>
            Failed ("check_kind_for_implicit_signal_chain", Target);
      end case;
   end Check_Kind_For_Implicit_Signal_Chain;

   function Get_Implicit_Signal_Chain (Target : Iir) return Iir is
   begin
      Check_Kind_For_Implicit_Signal_Chain (Target);
      return Target.Field5;
   end Get_Implicit_Signal_Chain;

   procedure Set_Implicit_Signal_Chain (Target : Iir; Sig : Iir) is
   begin
      Check_Kind_For_Implicit_Signal_Chain (Target);
      Target.Field5 := Sig;
   end Set_Implicit_Signal_Chain;

   procedure Check_Kind_For_Mode (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Interface_Declaration
           | Iir_Kind_File_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Mode;

   procedure Set_Mode (Target: Iir; Mode : Iir_Mode) is
   begin
      Check_Kind_For_Mode (Target);
      Target.Odigit_2 := Iir_Mode'Pos (Mode);
   end Set_Mode;

   function Get_Mode (Target: Iir) return Iir_Mode is
   begin
      Check_Kind_For_Mode (Target);
      return Iir_Mode'Val (Target.Odigit_2);
   end Get_Mode;

   procedure Check_Kind_For_Lexical_Layout (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Interface_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Lexical_Layout;

   procedure Set_Lexical_Layout (Target : Iir;
                                 Layout : Iir_Lexical_Layout_Type) is
   begin
      Check_Kind_For_Lexical_Layout (Target);
      Target.Odigit_1 := Iir_Lexical_Layout_Type'Pos (Layout);
   end Set_Lexical_Layout;

   function Get_Lexical_Layout (Target : Iir) return Iir_Lexical_Layout_Type is
   begin
      Check_Kind_For_Lexical_Layout (Target);
      return Bit3_Type'Pos (Target.Odigit_1);
   end Get_Lexical_Layout;

   procedure Check_Kind_For_Interface_Declaration (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Interface_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Interface_Declaration;

   procedure Check_Kind_For_Identifier (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Design_Unit
           | Iir_Kind_Architecture_Declaration
           | Iir_Kind_Configuration_Declaration
           | Iir_Kind_Package_Declaration
           | Iir_Kind_Entity_Declaration
           | Iir_Kind_Package_Body
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Type_Declaration
           | Iir_Kind_Subtype_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration
           | Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Procedure_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_File_Declaration
           | Iir_Kind_Element_Declaration
           | Iir_Kind_Attribute_Declaration
           | Iir_Kind_Library_Declaration
           | Iir_Kind_Library_Clause
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Group_Template_Declaration
           | Iir_Kind_Group_Declaration
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Physical_Unit
           | Iir_Kind_Simple_Name
           | Iir_Kind_Selected_Name
           | Iir_Kind_Character_Literal
           | Iir_Kind_String_Literal
           | Iir_Kind_Bit_String_Literal
           | Iir_Kind_Attribute
           | Iir_Kinds_Sequential_Statement
           | Iir_Kinds_Concurrent_Statement =>
            null;
         when others =>
            Failed ("identifier", Target);
      end case;
   end Check_Kind_For_Identifier;

   function Get_Identifier (Target: Iir) return Name_Id is
   begin
      Check_Kind_For_Identifier (Target);
      return Target.Identifier;
   end Get_Identifier;

   procedure Set_Identifier (Target: Iir; Identifier: Name_Id) is
   begin
      Check_Kind_For_Identifier (Target);
      Target.Identifier := Identifier;
   end Set_Identifier;

   procedure Check_Kind_For_Visible_Flag (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Design_Unit
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Type_Declaration
           | Iir_Kind_Subtype_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration
           | Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Procedure_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_File_Declaration
           | Iir_Kind_Element_Declaration
           | Iir_Kind_Attribute_Declaration
           | Iir_Kind_Library_Declaration
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Group_Template_Declaration
           | Iir_Kind_Group_Declaration
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Physical_Unit
           | Iir_Kinds_Sequential_Statement
           | Iir_Kinds_Concurrent_Statement =>
            null;
         when others =>
            Failed ("visible_flag", Target);
      end case;
   end Check_Kind_For_Visible_Flag;

   procedure Set_Visible_Flag (Target : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Visible_Flag (Target);
      Target.Flag4 := Flag;
   end Set_Visible_Flag;

   function Get_Visible_Flag (Target : Iir) return Boolean is
   begin
      Check_Kind_For_Visible_Flag (Target);
      return Target.Flag4;
   end Get_Visible_Flag;

   -- iir_kinds_object_declaration
   procedure Check_Kind_For_Default_Value (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Default_Value;

   procedure Set_Default_Value (Target: Iir; Value: Iir) is
   begin
      Check_Kind_For_Default_Value (Target);
      Target.Field2 := Value;
   end Set_Default_Value;

   -- iir_kinds_object_declaration
   function Get_Default_Value (Target: Iir) return Iir is
   begin
      Check_Kind_For_Default_Value (Target);
      return Target.Field2;
   end Get_Default_Value;

   procedure Set_Deferred_Declaration_Flag
     (Target : Iir_Constant_Declaration; Flag : Boolean) is
   begin
      Target.Flag1 := Flag;
   end Set_Deferred_Declaration_Flag;

   function Get_Deferred_Declaration_Flag (Target : Iir_Constant_Declaration)
     return Boolean is
   begin
      return Target.Flag1;
   end Get_Deferred_Declaration_Flag;

   procedure Set_Deferred_Declaration
     (Target : Iir_Constant_Declaration; Decl : Iir) is
   begin
      Target.Field3 := Decl;
   end Set_Deferred_Declaration;

   function Get_Deferred_Declaration (Target : Iir_Constant_Declaration)
     return Iir is
   begin
      return Target.Field3;
   end Get_Deferred_Declaration;

   procedure Check_Kind_For_Design_Unit (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Constant_Declaration
           | Iir_Kind_Entity_Declaration
           | Iir_Kind_Architecture_Declaration
           | Iir_Kind_Package_Declaration
           | Iir_Kind_Package_Body
           | Iir_Kind_Configuration_Declaration =>
            null;
         when others =>
            Failed ("check_kind_for_design_unit", Target);
      end case;
   end Check_Kind_For_Design_Unit;

   function Get_Design_Unit (Target : Iir) return Iir_Design_Unit is
   begin
      Check_Kind_For_Design_Unit (Target);
      return Target.Field5;
   end Get_Design_Unit;

   procedure Set_Design_Unit (Target : Iir; Unit : Iir_Design_Unit) is
   begin
      Check_Kind_For_Design_Unit (Target);
      Target.Field5 := Unit;
   end Set_Design_Unit;

   procedure Set_Shared_Flag (Target : Iir_Variable_Declaration;
                              Shared : Boolean)
   is
   begin
      Target.Flag2 := Shared;
   end Set_Shared_Flag;

   function Get_Shared_Flag (Target : Iir_Variable_Declaration)
     return Boolean is
   begin
      return Target.Flag2;
   end Get_Shared_Flag;

   function Get_Multiplier (Target: Iir_Physical_Unit) return Iir is
   begin
      return Target.Field2;
   end Get_Multiplier;

   procedure Set_Multiplier (Target: Iir_Physical_Unit; Val: Iir) is
   begin
      Target.Field2 := Val;
   end Set_Multiplier;

   procedure Check_Kind_For_Guard_Signal_Declaration  (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Guard_Signal_Declaration =>
            null;
         when others =>
            Failed ("guard_signal_declaration", Target);
      end case;
   end Check_Kind_For_Guard_Signal_Declaration;

   procedure Set_Block_Statement (Target: Iir; Block: Iir_Block_Statement) is
   begin
      Check_Kind_For_Guard_Signal_Declaration (Target);
      Target.Field3 := Block;
   end Set_Block_Statement;

   function Get_Block_Statement (Target: Iir) return Iir_Block_Statement is
   begin
      Check_Kind_For_Guard_Signal_Declaration (Target);
      return Target.Field3;
   end Get_Block_Statement;

   procedure Check_Kind_For_Signal_Declaration (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Signal_Declaration =>
            null;
         when others =>
            Failed ("signal_declaration", Target);
      end case;
   end Check_Kind_For_Signal_Declaration;

   procedure Set_Signal_Driver (Target : Iir_Signal_Declaration; Driver : Iir)
   is
   begin
      Target.Field5 := Driver;
   end Set_Signal_Driver;

   function Get_Signal_Driver (Target : Iir_Signal_Declaration) return Iir is
   begin
      --Check_Kind_For_Signal_Declaration (Target);
      return Target.Field5;
   end Get_Signal_Driver;

   -- iir_library_unit
   function Get_Declaration_List (Target: Iir)
     return Iir_Declaration_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Declaration
           | Iir_Kind_Architecture_Declaration
           | Iir_Kind_Configuration_Declaration
           | Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Process_Statement
           | Iir_Kind_Package_Declaration
           | Iir_Kind_Package_Body
           | Iir_Kind_Block_Statement
           | Iir_Kind_Generate_Statement
           | Iir_Kind_Block_Configuration =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field3;
   end Get_Declaration_List;

   procedure Set_Declaration_List (Target: Iir; Decls : Iir_Declaration_List)
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Block_Configuration =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field3 := Decls;
   end Set_Declaration_List;

   -- iir_use_clause
   procedure Set_Selected_Name (Target: Iir_Use_Clause; Name: Iir) is
   begin
      Target.Field1 := Name;
   end Set_Selected_Name;

   function Get_Selected_Name (Target: Iir_Use_Clause) return Iir is
   begin
      return Target.Field1;
   end Get_Selected_Name;


   procedure Check_Kind_For_Concurrent_Statement_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Architecture_Declaration
           | Iir_Kind_Block_Statement
           | Iir_Kind_Generate_Statement
           | Iir_Kind_Entity_Declaration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Concurrent_Statement_List;

   procedure Set_Concurrent_Statement_List
     (Target: Iir; List: Iir_Concurrent_Statement_List) is
   begin
      Check_Kind_For_Concurrent_Statement_List (Target);
      Target.Field2 := List;
   end Set_Concurrent_Statement_List;

   function Get_Concurrent_Statement_List
     (Target: Iir)
      return Iir_Concurrent_Statement_List is
   begin
      Check_Kind_For_Concurrent_Statement_List (Target);
      return Target.Field2;
   end Get_Concurrent_Statement_List;

   -- iir_kinds_subprogram_declaration
   function Get_Interface_Declaration (Target: Iir)
     return Iir_Interface_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Procedure_Declaration =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field4;
   end Get_Interface_Declaration;

   procedure Set_Interface_Declaration
     (Target : Iir_Implicit_Function_Declaration; List : Iir_Interface_List) is
   begin
      Target.Field4 := List;
   end Set_Interface_Declaration;

   procedure Add_Subprogram_Body (Target: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Spec_Body
           | Iir_Kind_Procedure_Spec_Body =>
            null;
         when others =>
            Failed ("add_subprogram_body", Target);
      end case;
      Target.Field2 := new Iir_Node (Iir_Kind_Declaration_List);
      Target.Field3 := new Iir_Node (Iir_Kind_Sequential_Statement_List);
   end Add_Subprogram_Body;

   procedure Copy_Subprogram_Body (Target: in out Iir; Src: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Function_Declaration
           | Iir_Kinds_Procedure_Declaration =>
            null;
         when others =>
            Failed ("copy_subprogram_body", Target);
      end case;
      if Get_Kind (Target) /= Get_Kind (Src) then
         raise Errorout.Internal_Error;
      end if;
      Target.Field2 := Src.Field2;
      Target.Field3 := Src.Field3;
   end Copy_Subprogram_Body;

   function Get_Subprogram_Declaration (Target: Iir) return Iir_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Spec_Body
           | Iir_Kind_Function_Body
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Procedure_Body =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Subprogram_Declaration;

   function Get_Subprogram_Statements (Target: Iir) return Iir_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Spec_Body
           | Iir_Kind_Function_Body
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Procedure_Body =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field3;
   end Get_Subprogram_Statements;

   procedure Check_Kind_For_Subprogram_Specification (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Body
           | Iir_Kind_Procedure_Body =>
            null;
         when others =>
            Failed ("check_kind_for_subprogram_specification", Target);
      end case;
   end Check_Kind_For_Subprogram_Specification;

   function Get_Subprogram_Specification (Target: Iir) return Iir is
   begin
      Check_Kind_For_Subprogram_Specification (Target);
      return Target.Field4;
   end Get_Subprogram_Specification;

   procedure Set_Subprogram_Specification (Target: Iir; Spec : Iir) is
   begin
      Check_Kind_For_Subprogram_Specification (Target);
      Target.Field4 := Spec;
   end Set_Subprogram_Specification;

   function Get_Subprogram_Body (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Specification
           | Iir_Kind_Procedure_Specification =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field3;
   end Get_Subprogram_Body;

   procedure Set_Subprogram_Body (Target: Iir; A_Body: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Specification
           | Iir_Kind_Procedure_Specification =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field3 := A_Body;
   end Set_Subprogram_Body;

   procedure Check_Kind_For_Function_Specification (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Function_Specification =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Function_Specification;

   procedure Check_Kind_For_Return_Type (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Function_Specification
           | Iir_Kind_Enumeration_Literal =>
            null;
         when others =>
            Failed ("check_kind_for_return_type", Target);
      end case;
   end Check_Kind_For_Return_Type;

   procedure Set_Return_Type (Target: Iir; Decl: Iir) is
   begin
      Check_Kind_For_Return_Type (Target);
      Target.Field1 := Decl;
   end Set_Return_Type;

   function Get_Return_Type (Target: Iir) return Iir is
   begin
      Check_Kind_For_Return_Type (Target);
      return Target.Field1;
   end Get_Return_Type;

   procedure Check_Kind_For_Implicit_Subprogram (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Implicit_Procedure_Declaration =>
            null;
         when others =>
            Failed ("implicit_subprogram");
      end case;
   end Check_Kind_For_Implicit_Subprogram;

   procedure Set_Implicit_Definition  (Target : Iir;
                                       Def : Iir_Predefined_Functions) is
   begin
      Check_Kind_For_Implicit_Subprogram (Target);
      Target.Nbr3 := Iir_Predefined_Functions'Pos (Def);
   end Set_Implicit_Definition;

   function Get_Implicit_Definition (Target : Iir)
     return Iir_Predefined_Functions is
   begin
      Check_Kind_For_Implicit_Subprogram (Target);
      return Iir_Predefined_Functions'Val (Target.Nbr3);
   end Get_Implicit_Definition;

   -----------------
   -- Definitions --
   -----------------

   function Get_Enumeration_Literal_List
     (Def: Iir_Enumeration_Type_Definition) return Iir is
   begin
      return Def.field3;
   end Get_Enumeration_Literal_List;

   -- iir_kinds_type_definition
   procedure Check_Kind_For_Type_Declarator (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Enumeration_Type_Definition
           | Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Integer_Type_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Type_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Physical_Type_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Record_Type_Definition
           | Iir_Kind_Record_Subtype_Definition
           | Iir_Kind_Array_Type_Definition
           | Iir_Kind_Array_Subtype_Definition
           | Iir_Kind_Unconstrained_Array_Subtype_Definition
           | Iir_Kind_Access_Type_Definition
           | Iir_Kind_Access_Subtype_Definition
           | Iir_Kind_Incomplete_Type_Definition
           | Iir_Kind_File_Type_Definition =>
            null;
         when others =>
            Failed ("check_kind_for_type_declarator", Target);
      end case;
   end Check_Kind_For_Type_Declarator;

   procedure Set_Type_Declarator (Target: Iir; Decl: Iir) is
   begin
      Check_Kind_For_Type_Declarator (Target);
      Target.Field1 := Decl;
   end Set_Type_Declarator;

   function Get_Type_Declarator (Target: Iir) return Iir is
   begin
      Check_Kind_For_Type_Declarator (Target);
      return Target.Field1;
   end Get_Type_Declarator;

   -- iir_integer_type_definition
   function Get_Direction (Decl: Iir) return Iir_Direction is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      return Iir_Direction'Val (Decl.State1);
   end Get_Direction;

   procedure Set_Direction (Decl: Iir; Dir: Iir_Direction) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      Decl.State1 := Iir_Direction'Pos (Dir);
   end Set_Direction;

   function Get_Left_Limit (Decl: Iir_Range_Expression) return Iir is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      return Decl.Field2;
   end Get_Left_Limit;

   procedure Set_Left_Limit (Decl: Iir_Range_Expression;
                             Limit: Iir) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      Decl.Field2 := Limit;
   end Set_Left_Limit;

   function Get_Right_Limit (Decl: Iir_Range_Expression) return Iir is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      return Decl.Field3;
   end Get_Right_Limit;

   procedure Set_Right_Limit (Decl: Iir_Range_Expression;
                              Limit: Iir) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Range_Expression =>
            null;
         when others =>
            Failed;
      end case;
      Decl.Field3 := Limit;
   end Set_Right_Limit;

   function Get_Range_Constraint (Target : Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Enumeration_Type_Definition
           | Iir_Kind_Subtype_Definition =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Range_Constraint;

   procedure Set_Range_Constraint (Target : Iir; Constraint : Iir)
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Enumeration_Type_Definition
           | Iir_Kind_Subtype_Definition =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field2 := Constraint;
   end Set_Range_Constraint;

   procedure Check_Kind_For_Type_Or_Subtype (Decl : Iir) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Enumeration_Type_Definition
           | Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Integer_Type_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Type_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Record_Type_Definition
           | Iir_Kind_Record_Subtype_Definition
           | Iir_Kind_Array_Type_Definition
           | Iir_Kind_Array_Subtype_Definition
           | Iir_Kind_Unconstrained_Array_Subtype_Definition
           | Iir_Kind_Physical_Type_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Subtype_Definition
           | Iir_Kind_Access_Type_Definition
           | Iir_Kind_Access_Subtype_Definition
           | Iir_Kind_Incomplete_Type_Definition
           | Iir_Kind_File_Type_Definition =>
            null;
         when others =>
            Failed ("check_kind_for_type_or_subtype", Decl);
      end case;
   end Check_Kind_For_Type_Or_Subtype;

   function Get_Base_Type (Decl: Iir) return Iir is
   begin
      Check_Kind_For_Type_Or_Subtype (Decl);
      return Decl.Field4;
   end Get_Base_Type;

   procedure Set_Base_Type (Decl: Iir; Base_Type: Iir) is
   begin
      Check_Kind_For_Type_Or_Subtype (Decl);
      Decl.Field4 := Base_Type;
   end Set_Base_Type;

   procedure Check_Kind_For_Subtype (Atype : Iir) is
   begin
      case Get_Kind (Atype) is
         when Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Array_Subtype_Definition
           | Iir_Kind_Unconstrained_Array_Subtype_Definition
           | Iir_Kind_Record_Subtype_Definition
           | Iir_Kind_Subtype_Definition =>
            null;
         when others =>
            Failed ("check_kind_for_subtype", Atype);
      end case;
   end Check_Kind_For_Subtype;

   function Get_Resolution_Function (Decl: Iir) return Iir is
   begin
      Check_Kind_For_Subtype (Decl);
      return Decl.Field5;
   end Get_Resolution_Function;

   procedure Set_Resolution_Function (Decl: Iir; Func: Iir) is
   begin
      Check_Kind_For_Subtype (Decl);
      Decl.Field5 := Func;
   end Set_Resolution_Function;

   function Get_Text_File_Flag (Atype : Iir_File_Type_Definition)
     return Boolean is
   begin
      return Atype.Flag3;
   end Get_Text_File_Flag;

   procedure Set_Text_File_Flag (Atype : Iir_File_Type_Definition;
                                 Flag : Boolean) is
   begin
      Atype.Flag3 := Flag;
   end Set_Text_File_Flag;

   procedure Set_Primary_Unit (Target: Iir_Physical_Type_Definition;
                               Unit: Iir_Physical_Unit) is
   begin
      Target.Field2 := Unit;
   end Set_Primary_Unit;

   function Get_Primary_Unit (Target: Iir_Physical_Type_Definition)
                              return Iir_Physical_Unit is
   begin
      return Target.Field2;
   end Get_Primary_Unit;

   function Get_Unit_List (Target: Iir_Physical_Type_Definition)
                           return Iir_Unit_List is
   begin
      return Target.Field3;
   end Get_Unit_List;

   procedure Set_Type_Staticness (Target: Iir; Static: Iir_Staticness)
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Type_And_Subtype_Definition =>
            null;
         when others =>
            Failed ("set_type_staticness", Target);
      end case;
      Target.Staticness_1 := Static;
   end Set_Type_Staticness;

   function Get_Type_Staticness (Target: Iir) return Iir_Staticness is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Type_And_Subtype_Definition =>
            null;
         when others =>
            Failed ("get_type_staticness", Target);
      end case;
      return Target.Staticness_1;
   end Get_Type_Staticness;

   procedure Check_Kind_For_Index_List (Decl : Iir) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Array_Type_Definition
           | Iir_Kind_Array_Subtype_Definition
           | Iir_Kind_Indexed_Name =>
            null;
         when others =>
            Failed ("get_index_list", Decl);
      end case;
   end Check_Kind_For_Index_List;

   -- iir_array_type_definition.
   -- iir_array_subtype_definition.
   function Get_Index_List (Decl: Iir) return Iir_List is
   begin
      Check_Kind_For_Index_List (Decl);
      return Decl.Field2;
   end Get_Index_List;

   procedure Set_Index_List (Decl: Iir; List : Iir_List) is
   begin
      Check_Kind_For_Index_List (Decl);
      Decl.Field2 := List;
   end Set_Index_List;

   function Get_Element_Subtype (Decl: Iir) return Iir is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Array_Type_Definition
           | Iir_Kind_Array_Subtype_Definition =>
            null;
         when others =>
            Failed;
      end case;
      return Decl.Field3;
   end Get_Element_Subtype;

   procedure Set_Element_Subtype (Decl: Iir; Sub_Type: Iir) is
   begin
      case Get_Kind (Decl) is
         when Iir_Kind_Array_Type_Definition
           | Iir_Kind_Array_Subtype_Definition =>
            null;
         when others =>
            Failed;
      end case;
      Decl.Field3 := Sub_Type;
   end Set_Element_Subtype;

   function Get_Element_Declaration_List
     (Decl: Iir_Record_Type_Definition)
      return Iir_Element_Declaration_List is
   begin
      return Decl.Field2;
   end Get_Element_Declaration_List;

   procedure Set_Element_Position
     (Target: Iir_Element_Declaration; Pos: Iir_Index32) is
   begin
      Target.Nbr2 := Natural (Pos);
   end Set_Element_Position;

   function Get_Element_Position (Target: Iir_Element_Declaration)
     return Iir_Index32 is
   begin
      return Iir_Index32 (Target.Nbr2);
   end Get_Element_Position;

   procedure Check_Kind_For_Designated_Type (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Access_Type_Definition =>
            null;
         when others =>
            Failed ("check_kind_for_designated_type", Target);
      end case;
   end Check_Kind_For_Designated_Type;

   function Get_Designated_Type (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Access_Type_Definition =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Designated_Type;

   procedure Set_Designated_Type (Target: Iir; Dtype: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Access_Type_Definition =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field2 := Dtype;
   end Set_Designated_Type;

   function Get_Incomplete_Type_List (Target : Iir) return Iir_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Incomplete_Type_Definition =>
            null;
         when others =>
            Failed ("get_incomplete_type_list", Target);
      end case;
      return Target.Field2;
   end Get_Incomplete_Type_List;

   procedure Set_File_Logical_Name (Target: Iir_File_Declaration;
                                    Name: Iir) is
   begin
      Target.Field2 := Name;
   end Set_File_Logical_Name;

   function Get_File_Logical_Name (Target: Iir_File_Declaration)
                                   return Iir is
   begin
      return Target.Field2;
   end Get_File_Logical_Name;

   procedure Set_File_Open_Kind (Target: Iir_File_Declaration;
                                 Kind: Iir) is
   begin
      Target.Field3 := Kind;
   end Set_File_Open_Kind;

   function Get_File_Open_Kind (Target: Iir_File_Declaration)
                                return Iir is
   begin
      return Target.Field3;
   end Get_File_Open_Kind;

   function Get_Entity_Class_Entry_List
     (Group : Iir_Group_Template_Declaration)
      return Iir_Entity_Class_Entry_List is
   begin
      return Group.Field1;
   end Get_Entity_Class_Entry_List;

   function Get_Group_Constituent_List (Group : Iir_Group_Declaration)
     return Iir_Group_Constituent_List
   is
   begin
      return Group.Field1;
   end Get_Group_Constituent_List;

   procedure Set_Selected_Element (Target: Iir_Selected_Element;
                                   El : Iir_Element_Declaration) is
   begin
      Target.Field2 := El;
   end Set_Selected_Element;

   function Get_Selected_Element (Target: Iir_Selected_Element)
                                  return Iir_Element_Declaration is
   begin
      return Target.Field2;
   end Get_Selected_Element;

   ---------------------------
   -- concurrent statements --
   ---------------------------

   -- Iir_Concurrent_Conditional_Signal_Assignment
   procedure Check_Kind_For_Target (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Signal_Assignment_Statement
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Variable_Assignment_Statement =>
            null;
         when others =>
            Failed ("check_kind_for_target", Target);
      end case;
   end Check_Kind_For_Target;

   procedure Set_Target (Target: Iir; Atarget: Iir) is
   begin
      Check_Kind_For_Target (Target);
      Target.Field2 := Atarget;
   end Set_Target;

   function Get_Target (Target: Iir) return Iir is
   begin
      Check_Kind_For_Target (Target);
      return Target.Field2;
   end Get_Target;

   procedure Check_Kind_For_Waveform_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Conditional_Waveform
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("check_kind_for_waveform_list", Target);
      end case;
   end Check_Kind_For_Waveform_List;

   function Get_Waveform_List (Target: Iir) return Iir_Waveform_List is
   begin
      Check_Kind_For_Waveform_List (Target);
      return Target.Field3;
   end Get_Waveform_List;

   procedure Set_Waveform_List (Target: Iir; List: Iir_Waveform_List) is
   begin
      Check_Kind_For_Waveform_List (Target);
      Target.Field3 := List;
   end Set_Waveform_List;

   procedure Check_Kind_For_Guard (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Concurrent_Selected_Signal_Assignment =>
            null;
         when others =>
            Failed ("check_kind_for_guard", Target);
      end case;
   end Check_Kind_For_Guard;

   function Get_Guard (Target: Iir) return Iir is
   begin
      Check_Kind_For_Guard (Target);
      return Target.Field4;
   end Get_Guard;

   procedure Set_Guard (Target: Iir; Guard: Iir) is
   begin
      Check_Kind_For_Guard (Target);
      Target.Field4 := Guard;
   end Set_Guard;

   procedure Check_Kind_For_Signal_Assignment (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("signal_assignment", Target);
      end case;
   end Check_Kind_For_Signal_Assignment;

   function Get_Guarded_Target_State (Stmt : Iir) return Tri_State_Type is
   begin
      Check_Kind_For_Signal_Assignment (Stmt);
      return Tri_State_Type'Val (Stmt.State1);
   end Get_Guarded_Target_State;

   procedure Set_Guarded_Target_State (Stmt : Iir; State : Tri_State_Type) is
   begin
      Check_Kind_For_Signal_Assignment (Stmt);
      Stmt.State1 := Tri_State_Type'Pos (State);
   end Set_Guarded_Target_State;

   function Get_Delay_Mechanism (Target: Iir) return Iir_Delay_Mechanism is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("get_delay_mechanism", Target);
      end case;
      return Iir_Delay_Mechanism'Val (Target.Nbr3);
   end Get_Delay_Mechanism;

   procedure Set_Delay_Mechanism
     (Target: Iir; Delay_Mechanism: Iir_Delay_Mechanism) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("set_delay_mechanism", Target);
      end case;
      Target.Nbr3 := Iir_Delay_Mechanism'Pos (Delay_Mechanism);
   end Set_Delay_Mechanism;

   function Get_Reject_Time_Expression (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("get_reject_time_expression", Target);
      end case;
      return Target.Field5;
   end Get_Reject_Time_Expression;

   procedure Set_Reject_Time_Expression (Target: Iir; Expr: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Signal_Assignment_Statement =>
            null;
         when others =>
            Failed ("set_reject_time_expression", Target);
      end case;
      Target.Field5 := Expr;
   end Set_Reject_Time_Expression;

   procedure Check_Kind_For_Label (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Sequential_Statement
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Process_Statement
           | Iir_Kind_Component_Instantiation_Statement
           | Iir_Kind_Concurrent_Assertion_Statement
           | Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Concurrent_Procedure_Call_Statement
           | Iir_Kind_Block_Statement
           | Iir_Kind_Generate_Statement =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Label;

   procedure Set_Label (Target: Iir; Label: Name_Id) is
   begin
      Check_Kind_For_Label (Target);
      Target.Identifier := Label;
   end Set_Label;

   function Get_Label (Target: Iir) return Name_Id is
   begin
      Check_Kind_For_Label (Target);
      return Target.Identifier;
   end Get_Label;

   procedure Check_Kind_For_Expression (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Variable_Assignment_Statement
           | Iir_Kind_Return_Statement
           | Iir_Kind_Case_Statement
           | Iir_Kind_Qualified_Expression
           | Iir_Kind_Type_Conversion
           | Iir_Kind_Attribute_Specification
           | Iir_Kind_Allocator_By_Expression
           | Iir_Kind_Allocator_By_Subtype
           | Iir_Kind_Association_Choice_By_Expression
           | Iir_Kind_Association_Choice_By_Range
           | Iir_Kind_Disconnection_Specification =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Expression;

   -- iir_kind_concurrent_selected_signal_assignment.
   procedure Set_Expression (Target: Iir; Expr: Iir) is
   begin
      Check_Kind_For_Expression (Target);
      Target.Field3 := Expr;
   end Set_Expression;

   function Get_Expression (Target: Iir) return Iir is
   begin
      Check_Kind_For_Expression (Target);
      return Target.Field3;
   end Get_Expression;

   function Get_Selected_Waveform_List (Target: Iir)
     return Iir_Association_Choices_List is
   begin
      if Get_Kind (Target) /= Iir_Kind_Concurrent_Selected_Signal_Assignment
      then
         Failed ("get_selected_waveform_list", Target);
      end if;
      return Target.Field1;
   end Get_Selected_Waveform_List;

   procedure Check_Kind_For_Concurrent_Conditional_Signal_Assignment
     (Target : Iir)
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Concurrent_Conditional_Signal_Assignment =>
            null;
         when others =>
            Failed ("Concurrent_Conditional_Signal_Assignment", Target);
      end case;
   end Check_Kind_For_Concurrent_Conditional_Signal_Assignment;

   function Get_Conditional_Waveform_List (Target: Iir) return Iir_List is
   begin
      Check_Kind_For_Concurrent_Conditional_Signal_Assignment (Target);
      return Target.Field3;
   end Get_Conditional_Waveform_List;

   procedure Set_Conditional_Waveform_List (Target: Iir; List : Iir_List) is
   begin
      Check_Kind_For_Concurrent_Conditional_Signal_Assignment (Target);
      Target.Field3 := List;
   end Set_Conditional_Waveform_List;

   procedure Set_Guard_Expression
     (Target: Iir_Guard_Signal_Declaration; Expr: Iir) is
   begin
      Target.Field2 := Expr;
   end Set_Guard_Expression;

   function Get_Guard_Expression (Target: Iir_Guard_Signal_Declaration)
     return Iir is
   begin
      return Target.Field2;
   end Get_Guard_Expression;

   function Get_Guard_Decl (Target: Iir_Block_Statement)
     return Iir_Guard_Signal_Declaration is
   begin
      return Target.Field1;
   end Get_Guard_Decl;

   procedure Set_Guard_Decl (Target: Iir_Block_Statement;
                             Decl: Iir_Guard_Signal_Declaration) is
   begin
      Target.Field1 := Decl;
   end Set_Guard_Decl;

   procedure Set_Guard_Sensitivity_List
     (Guard : Iir_Guard_Signal_Declaration; List: Iir_Designator_List) is
   begin
      Guard.Field5 := List;
   end Set_Guard_Sensitivity_List;

   function Get_Guard_Sensitivity_List (Guard : Iir_Guard_Signal_Declaration)
     return Iir_Designator_List is
   begin
      return Guard.Field5;
   end Get_Guard_Sensitivity_List;

   procedure Set_Block_Header (Block: Iir_Block_Statement;
                               Header: Iir_Block_Header) is
   begin
      Set_Field7 (Block, Header);
   end Set_Block_Header;

   function Get_Block_Header (Block: Iir_Block_Statement)
                              return Iir_Block_Header is
   begin
      return Get_Field7 (Block);
   end Get_Block_Header;

   procedure Set_Block_Block_Configuration
     (Block : Iir_Block_Statement; Conf : Iir_Block_Configuration) is
   begin
      Block.Field4 := Conf;
   end Set_Block_Block_Configuration;

   function Get_Block_Block_Configuration (Block : Iir_Block_Statement)
     return Iir_Block_Configuration is
   begin
      return Block.Field4;
   end Get_Block_Block_Configuration;

   procedure Set_Generation_Scheme
     (Block : Iir_Generate_Statement; Scheme : Iir)
   is
   begin
      Block.Field5 := Scheme;
   end Set_Generation_Scheme;

   function Get_Generation_Scheme (Block : Iir_Generate_Statement)
     return Iir
   is
   begin
      return Block.Field5;
   end Get_Generation_Scheme;

   procedure Set_Generate_Block_Configuration
     (Block : Iir_Generate_Statement;
      Conf : Iir_Block_Configuration)
   is
   begin
      Block.Field1 := Conf;
   end Set_Generate_Block_Configuration;

   function Get_Generate_Block_Configuration (Block : Iir_Generate_Statement)
     return Iir_Block_Configuration
   is
   begin
      return Block.Field1;
   end Get_Generate_Block_Configuration;

   -----------------
   -- Expressions --
   -----------------

   procedure Check_Kind_For_Expr_Staticness (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Monadic_Operator
           | Iir_Kinds_Dyadic_Operator
           | Iir_Kinds_Name
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_File_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kinds_Type_Attribute
           | Iir_Kinds_Scalar_Type_Attribute
           | Iir_Kinds_Array_Attribute
           | Iir_Kind_Image_Attribute
           | Iir_Kind_Value_Attribute
           | Iir_Kind_Aggregate
           | Iir_Kind_Qualified_Expression
           | Iir_Kind_Type_Conversion
           | Iir_Kinds_Literal
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Simple_Aggregate
           | Iir_Kind_Range_Expression
           | Iir_Kind_Selected_Element
           | Iir_Kind_Attribute_Value
           | Iir_Kind_Function_Call
           | Iir_Kinds_Signal_Attribute
           | Iir_Kinds_Signal_Value_Attribute
           | Iir_Kind_Attribute
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference
           | Iir_Kind_Physical_Unit
           | Iir_Kind_Allocator_By_Expression
           | Iir_Kind_Allocator_By_Subtype =>
            null;
         when others =>
            Failed ("expr_staticness", Target);
      end case;
   end Check_Kind_For_Expr_Staticness;

   procedure Set_Expr_Staticness (Target: Iir; Static: Iir_Staticness)
   is
   begin
      Check_Kind_For_Expr_Staticness (Target);
      Target.Staticness_1 := Static;
   end Set_Expr_Staticness;

   -- See 7.4.1
   function Get_Expr_Staticness (Target: Iir) return Iir_Staticness is
   begin
      Check_Kind_For_Expr_Staticness (Target);
      return Target.Staticness_1;
   end Get_Expr_Staticness;

   -- iir_monadic_operator
   procedure Set_Operand (Target: Iir; An_Iir: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Monadic_Operator =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field2 := An_Iir;
   end Set_Operand;

   function Get_Operand (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Monadic_Operator =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Operand;

   -- iir_dyadic_operator
   procedure Set_Left (Target: Iir; An_Iir: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Dyadic_Operator =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field2 := An_Iir;
   end Set_Left;

   function Get_Left (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Dyadic_Operator =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Left;

   procedure Check_Kind_For_Right (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Dyadic_Operator =>
            null;
         when others =>
            Failed ("right", Target);
      end case;
   end Check_Kind_For_Right;

   procedure Set_Right (Target: Iir; An_Iir: Iir) is
   begin
      Check_Kind_For_Right (Target);
      Target.Field4 := An_Iir;
   end Set_Right;

   function Get_Right (Target: Iir) return Iir is
   begin
      Check_Kind_For_Right (Target);
      return Target.Field4;
   end Get_Right;

   procedure Check_Kind_For_Implementation (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Call
           | Iir_Kind_Procedure_Call
           | Iir_Kind_Concurrent_Procedure_Call_Statement
           | Iir_Kinds_Dyadic_Operator
           | Iir_Kinds_Monadic_Operator =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Implementation;

   procedure Set_Implementation (Target: Iir; Decl: Iir) is
   begin
      Check_Kind_For_Implementation (Target);
      Target.Field3 := Decl;
   end Set_Implementation;

   function Get_Implementation (Target: Iir) return Iir is
   begin
      Check_Kind_For_Implementation (Target);
      return Target.Field3;
   end Get_Implementation;

   procedure Check_Kind_For_Parameter_Association_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Function_Call
           | Iir_Kind_Procedure_Call
           | Iir_Kind_Concurrent_Procedure_Call_Statement =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Parameter_Association_List;

   procedure Set_Parameter_Association_List
     (Target: Iir; List: Iir_Association_List)
   is
   begin
      Check_Kind_For_Parameter_Association_List (Target);
      Target.Field2 := List;
   end Set_Parameter_Association_List;

   function Get_Parameter_Association_List (Target: Iir)
     return Iir_Association_List
   is
   begin
      Check_Kind_For_Parameter_Association_List (Target);
      return Target.Field2;
   end Get_Parameter_Association_List;

   procedure Check_Kind_For_Type_Mark (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Qualified_Expression
           | Iir_Kind_File_Type_Definition
           | Iir_Kind_Integer_Subtype_Definition
           | Iir_Kind_Floating_Subtype_Definition
           | Iir_Kind_Enumeration_Subtype_Definition
           | Iir_Kind_Physical_Subtype_Definition
           | Iir_Kind_Access_Subtype_Definition
           | Iir_Kind_Record_Subtype_Definition
           | Iir_Kind_Array_Subtype_Definition
         -- | Iir_Kind_File_Subtype_Definition
           | Iir_Kind_Unconstrained_Array_Subtype_Definition =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Type_Mark;

   procedure Set_Type_Mark (Target: Iir; Mark: Iir) is
   begin
      Check_Kind_For_Type_Mark (Target);
      Set_Field6 (Target, Mark);
   end Set_Type_Mark;

   function Get_Type_Mark (Target: Iir) return Iir is
   begin
      Check_Kind_For_Type_Mark (Target);
      return Get_Field6 (Target);
   end Get_Type_Mark;

   ----------------------------
   -- concurrent statements  --
   ----------------------------

   procedure Set_Sensitivity_List
     (Wait: Iir_Wait_Statement; List: Iir_Designator_List) is
   begin
      Wait.Field4 := List;
   end Set_Sensitivity_List;

   function Get_Sensitivity_List (Target: Iir) return Iir_Designator_List
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Wait_Statement =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field4;
   end Get_Sensitivity_List;

   procedure Check_Kind_For_Proc (Proc : Iir) is
   begin
      case Get_Kind (Proc) is
         when Iir_Kind_Process_Statement
           | Iir_Kind_Sensitized_Process_Statement
           | Iir_Kinds_Procedure_Specification
           | Iir_Kinds_Function_Specification =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Proc;

   function Get_Driver_List (Stmt: Iir) return Iir_Driver_List is
   begin
      Check_Kind_For_Proc (Stmt);
      return Stmt.Field5;
   end Get_Driver_List;

   procedure Set_Driver_List (Stmt: Iir; List: Iir_Driver_List) is
   begin
      Check_Kind_For_Proc (Stmt);
      Free_Iir (Stmt.Field5);
      Stmt.Field5 := List;
   end Set_Driver_List;

   procedure Set_Callees_List (Proc : Iir; List : Iir_Callees_List)
   is
   begin
      Check_Kind_For_Proc (Proc);
      Set_Field7 (Proc, List);
   end Set_Callees_List;

   function Get_Callees_List (Proc : Iir) return Iir_Callees_List is
   begin
      Check_Kind_For_Proc (Proc);
      return Get_Field7 (Proc);
   end Get_Callees_List;

   procedure Check_Kind_For_Process (Process : Iir) is
   begin
      case Get_Kind (Process) is
         when Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Process_Statement =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Process;

   procedure Set_Passive_Flag (Proc : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Process (Proc);
      Proc.Flag2 := Flag;
   end Set_Passive_Flag;

   function Get_Passive_Flag (Proc : Iir) return Boolean is
   begin
      Check_Kind_For_Process (Proc);
      return Proc.Flag2;
   end Get_Passive_Flag;

   procedure Set_Passive_State (Proc : Iir; State : Tri_State_Type) is
   begin
      Check_Kind_For_Proc (Proc);
      Proc.State1 := Tri_State_Type'Pos (State);
   end Set_Passive_State;

   function Get_Passive_State (Proc : Iir) return Tri_State_Type is
   begin
      Check_Kind_For_Proc (Proc);
      return Tri_State_Type'Val (Proc.State1);
   end Get_Passive_State;

   procedure Set_Wait_State (Proc : Iir; State : Tri_State_Type) is
   begin
      Check_Kind_For_Proc (Proc);
      Proc.State2 := State;
   end Set_Wait_State;

   function Get_Wait_State (Proc : Iir) return Tri_State_Type is
   begin
      Check_Kind_For_Proc (Proc);
      return Proc.State2;
   end Get_Wait_State;

   procedure Set_Seen_Flag (Proc : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Proc (Proc);
      Proc.Flag1 := Flag;
   end Set_Seen_Flag;

   function Get_Seen_Flag (Proc : Iir) return Boolean is
   begin
      Check_Kind_For_Proc (Proc);
      return Proc.Flag1;
   end Get_Seen_Flag;

   procedure Set_Resolved_Flag (Atype : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Type_Or_Subtype (Atype);
      Atype.Flag1 := Flag;
   end Set_Resolved_Flag;

   function Get_Resolved_Flag (Atype : Iir) return Boolean is
   begin
      Check_Kind_For_Type_Or_Subtype (Atype);
      return Atype.Flag1;
   end Get_Resolved_Flag;

   procedure Set_Signal_Type_Flag (Atype : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Type_Or_Subtype (Atype);
      Atype.Flag2 := Flag;
   end Set_Signal_Type_Flag;

   function Get_Signal_Type_Flag (Atype : Iir) return Boolean is
   begin
      Check_Kind_For_Type_Or_Subtype (Atype);
      return Atype.Flag2;
   end Get_Signal_Type_Flag;

   procedure Set_Pure_Flag (Func : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Function_Specification (Func);
      Func.Flag2 := Flag;
   end Set_Pure_Flag;

   function Get_Pure_Flag (Func : Iir) return Boolean is
   begin
      Check_Kind_For_Function_Specification (Func);
      return Func.Flag2;
   end Get_Pure_Flag;

   procedure Set_Purity_State (Proc : Iir; State : Tri_State_Type) is
   begin
      Check_Kind_For_Proc (Proc);
      Proc.State3 := State;
   end Set_Purity_State;

   function Get_Purity_State (Proc: Iir) return Tri_State_Type is
   begin
      Check_Kind_For_Proc (Proc);
      return Proc.State3;
   end Get_Purity_State;

   procedure Check_Kind_For_Elab (Design : Iir)
   is
   begin
      case Get_Kind (Design) is
         when Iir_Kind_Design_File
           | Iir_Kind_Design_Unit =>
            null;
         when others =>
            Failed ("design", Design);
      end case;
   end Check_Kind_For_Elab;

   procedure Set_Elab_Flag (Design : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Elab (Design);
      Design.Flag3 := Flag;
   end Set_Elab_Flag;

   function Get_Elab_Flag (Design : Iir) return Boolean is
   begin
      Check_Kind_For_Elab (Design);
      return Design.Flag3;
   end Get_Elab_Flag;

   function Get_Process_Statement_List (Process: Iir)
     return Iir_Sequential_Statement_List is
   begin
      Check_Kind_For_Process (Process);
      return Process.Field2;
   end Get_Process_Statement_List;

   procedure Check_Kind_For_Assertion_Condition (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Assertion_Statement
           | Iir_Kind_Concurrent_Assertion_Statement =>
            null;
         when others =>
            Failed ("assertion_condition", Target);
      end case;
   end Check_Kind_For_Assertion_Condition;

   procedure Set_Assertion_Condition (Target: Iir; Cond: Iir) is
   begin
      Check_Kind_For_Assertion_Condition (Target);
      Target.Field2 := Cond;
   end Set_Assertion_Condition;

   function Get_Assertion_Condition (Target: Iir) return Iir is
   begin
      Check_Kind_For_Assertion_Condition (Target);
      return Target.Field2;
   end Get_Assertion_Condition;

   procedure Check_Kind_For_Report_Expression (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Assertion_Statement
           | Iir_Kind_Report_Statement
           | Iir_Kind_Concurrent_Assertion_Statement =>
            null;
         when others =>
            Failed ("report_expression", Target);
      end case;
   end Check_Kind_For_Report_Expression;

   procedure Set_Report_Expression (Target: Iir; Expr: Iir) is
   begin
      Check_Kind_For_Report_Expression (Target);
      Target.Field3 := Expr;
   end Set_Report_Expression;

   function Get_Report_Expression (Target: Iir) return Iir is
   begin
      Check_Kind_For_Report_Expression (Target);
      return Target.Field3;
   end Get_Report_Expression;

   procedure Set_Severity_Expression (Target: Iir; Expr: Iir) is
   begin
      Check_Kind_For_Report_Expression (Target);
      Target.Field4 := Expr;
   end Set_Severity_Expression;

   function Get_Severity_Expression (Target: Iir) return Iir is
   begin
      Check_Kind_For_Report_Expression (Target);
      return Target.Field4;
   end Get_Severity_Expression;

   ---------------------------
   -- sequential statements --
   ---------------------------

   procedure Check_Kind_For_Condition (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_If_Statement
           | Iir_Kind_Elsif
           | Iir_Kind_Conditional_Waveform
           | Iir_Kind_While_Loop_Statement
           | Iir_Kind_Next_Statement
           | Iir_Kind_Exit_Statement =>
            null;
         when others =>
            Failed ("condition", Target);
      end case;
   end Check_Kind_For_Condition;

   procedure Set_Condition (Target: Iir; Condition: Iir) is
   begin
      Check_Kind_For_Condition (Target);
      Target.Field1 := Condition;
   end Set_Condition;

   function Get_Condition (Target: Iir) return Iir is
   begin
      Check_Kind_For_Condition (Target);
      return Target.Field1;
   end Get_Condition;

   function Get_Then_Sequence_List (Target: Iir)
     return Iir_Sequential_Statement_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_If_Statement
           | Iir_Kind_Elsif =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field2;
   end Get_Then_Sequence_List;

   procedure Set_Else_Clause (Target: Iir; Clause: Iir_Elsif) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_If_Statement
           | Iir_Kind_Elsif =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field3 := Clause;
   end Set_Else_Clause;

   function Get_Else_Clause (Target: Iir) return Iir_Elsif is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_If_Statement
           | Iir_Kind_Elsif =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field3;
   end Get_Else_Clause;

   procedure Set_Iterator_Scheme (Target: Iir_For_Loop_Statement;
                                  Iterator: Iir) is
   begin
      Target.Field1 := Iterator;
   end Set_Iterator_Scheme;

   function Get_Iterator_Scheme (Target: Iir_For_Loop_Statement)
                                 return Iir is
   begin
      return Target.Field1;
   end Get_Iterator_Scheme;

   procedure Set_Condition_Clause
     (Wait: Iir_Wait_Statement; Cond: Iir) is
   begin
      Wait.Field2 := Cond;
   end Set_Condition_Clause;

   function Get_Condition_Clause (Wait: Iir_Wait_Statement) return Iir is
   begin
      return Wait.Field2;
   end Get_Condition_Clause;

   procedure Set_Timeout_Clause
     (Wait: Iir_Wait_Statement; Timeout: Iir) is
   begin
      Wait.Field3 := Timeout;
   end Set_Timeout_Clause;

   function Get_Timeout_Clause (Wait: Iir_Wait_Statement) return Iir is
   begin
      return Wait.Field3;
   end Get_Timeout_Clause;

   function Get_Statement_List (Target: Iir) return Iir_List is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_For_Loop_Statement
           | Iir_Kind_While_Loop_Statement =>
            null;
         when others =>
            Failed ("get_statement_list", Target);
      end case;
      return Target.Field2;
   end Get_Statement_List;

   procedure Set_Parent (Target : Iir; Parent : Iir) is
   begin
      Target.Parent := Parent;
   end Set_Parent;

   function Get_Parent (Target : Iir) return Iir is
   begin
      return Target.Parent;
   end Get_Parent;

   procedure Check_Kind_For_Loop (Target: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Exit_Statement
           | Iir_Kind_Next_Statement =>
            null;
         when others =>
            Failed ("check_kind_for_loop", Target);
      end case;
   end Check_Kind_For_Loop;

   function Get_Loop (Target: Iir) return Iir is
   begin
      Check_Kind_For_Loop (Target);
      return Target.Field2;
   end Get_Loop;

   procedure Set_Loop (Target: Iir; Stmt: Iir) is
   begin
      Check_Kind_For_Loop (Target);
      Target.Field2 := Stmt;
   end Set_Loop;

   procedure Set_Component_Name (Target: Iir; Name: Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      Target.Field4 := Name;
   end Set_Component_Name;

   function Get_Component_Name (Target: Iir) return Iir is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field4;
   end Get_Component_Name;

   function Get_Instantiation_List (Target: Iir) return Iir
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      return Target.Field1;
   end Get_Instantiation_List;

   procedure Set_Instantiation_List (Target: Iir; List : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Configuration
           | Iir_Kind_Configuration_Specification =>
            null;
         when others =>
            Failed;
      end case;
      if Target.Field1 /= Null_Iir and List /= Null_Iir then
         raise Internal_Error;
      end if;
      Target.Field1 := List;
   end Set_Instantiation_List;

   procedure Check_Kind_For_Binding_Indication (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Binding_Indication =>
            null;
         when others =>
            Failed ("binding_indication");
      end case;
   end Check_Kind_For_Binding_Indication;

   procedure Set_Entity_Aspect (Target: Iir_Binding_Indication; Entity: Iir) is
   begin
      Check_Kind_For_Binding_Indication (Target);
      Target.Field4 := Entity;
   end Set_Entity_Aspect;

   function Get_Entity_Aspect (Target: Iir_Binding_Indication) return Iir  is
   begin
      Check_Kind_For_Binding_Indication (Target);
      return Target.Field4;
   end Get_Entity_Aspect;

   procedure Set_Default_Entity_Aspect (Target : Iir_Binding_Indication;
                                        Aspect : Iir) is
   begin
      Target.Field1 := Aspect;
   end Set_Default_Entity_Aspect;

   function Get_Default_Entity_Aspect (Target : Iir_Binding_Indication)
     return Iir is
   begin
      return Target.Field1;
   end Get_Default_Entity_Aspect;

   procedure Set_Default_Generic_Map_Aspect_List
     (Target : Iir_Binding_Indication; List : Iir_List) is
   begin
      Target.Field2 := List;
   end Set_Default_Generic_Map_Aspect_List;

   function Get_Default_Generic_Map_Aspect_List
     (Target : Iir_Binding_Indication)
     return Iir_List is
   begin
      return Target.Field2;
   end Get_Default_Generic_Map_Aspect_List;

   procedure Set_Default_Port_Map_Aspect_List (Target : Iir_Binding_Indication;
                                               List : Iir_List) is
   begin
      Set_Field6 (Target, List);
   end Set_Default_Port_Map_Aspect_List;

   function Get_Default_Port_Map_Aspect_List (Target : Iir_Binding_Indication)
     return Iir_List is
   begin
      return Get_Field6 (Target);
   end Get_Default_Port_Map_Aspect_List;

   function Create_Proxy (Proxy: Iir) return Iir_Proxy is
      Res : Iir_Proxy;
   begin
      Res := Create_Iir (Iir_Kind_Proxy);
      Set_Proxy (Res, Proxy);
      return Res;
   end Create_Proxy;

   function Get_Proxy (Proxy: Iir_Proxy) return Iir is
   begin
      return Proxy.Field1;
   end Get_Proxy;

   procedure Set_Proxy (Target: Iir_Proxy; Proxy: Iir) is
   begin
      Target.Field1 := Proxy;
   end Set_Proxy;

   -- iir_kinds_association_element
   procedure Check_Kind_For_Association_Element (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kinds_Association_Element =>
            null;
         when others =>
            Failed ("check_kind_for_association_element", Target);
      end case;
   end Check_Kind_For_Association_Element;

   procedure Set_Formal (Target: Iir; Formal: Iir) is
   begin
      Check_Kind_For_Association_Element (Target);
      Target.Field1 := Formal;
   end Set_Formal;

   function Get_Formal (Target: Iir) return Iir is
   begin
      Check_Kind_For_Association_Element (Target);
      return Target.Field1;
   end Get_Formal;

   procedure Check_Kind_For_Association_Element_By_Expr (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Association_Element_By_Expression =>
            null;
         when others =>
            Failed ("check_kind_for_association_element_by_expr", Target);
      end case;
   end Check_Kind_For_Association_Element_By_Expr;

   procedure Set_Actual (Target: Iir; Actual: Iir) is
   begin
      Check_Kind_For_Association_Element_By_Expr (Target);
      Target.Field2 := Actual;
   end Set_Actual;

   function Get_Actual (Target: Iir) return Iir is
   begin
      Check_Kind_For_Association_Element_By_Expr (Target);
      return Target.Field2;
   end Get_Actual;

   procedure Set_Associated_Formal (Target: Iir; Formal : Iir) is
   begin
      Check_Kind_For_Association_Element (Target);
      Target.Field3 := Formal;
   end Set_Associated_Formal;

   function Get_Associated_Formal (Target: Iir) return Iir is
   begin
      Check_Kind_For_Association_Element (Target);
      return Target.Field3;
   end Get_Associated_Formal;

   procedure Set_In_Conversion (Target : Iir; Conv : Iir) is
   begin
      Check_Kind_For_Association_Element (Target);
      Target.Field4 := Conv;
   end Set_In_Conversion;

   function Get_In_Conversion (Target : Iir) return Iir is
   begin
      Check_Kind_For_Association_Element (Target);
      return Target.Field4;
   end Get_In_Conversion;

   procedure Set_Out_Conversion (Target : Iir; Conv : Iir) is
   begin
      Check_Kind_For_Association_Element (Target);
      Target.Field5 := Conv;
   end Set_Out_Conversion;

   function Get_Out_Conversion (Target : Iir) return Iir is
   begin
      Check_Kind_For_Association_Element (Target);
      return Target.Field5;
   end Get_Out_Conversion;

   procedure Set_Whole_Association_Flag (Target : Iir; Flag : Boolean) is
   begin
      Check_Kind_For_Association_Element (Target);
      Target.Flag1 := Flag;
   end Set_Whole_Association_Flag;

   function Get_Whole_Association_Flag (Target : Iir) return Boolean is
   begin
      Check_Kind_For_Association_Element (Target);
      return Target.Flag1;
   end Get_Whole_Association_Flag;

   procedure Check_Kind_For_Instantiated_Unit (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Instantiation_Statement =>
            null;
         when others =>
            Failed ("instantiated_unit", Target);
      end case;
   end Check_Kind_For_Instantiated_Unit;

   procedure Set_Instantiated_Unit (Target: Iir; Unit: Iir) is
   begin
      Check_Kind_For_Instantiated_Unit (Target);
      Target.Field2 := Unit;
   end Set_Instantiated_Unit;

   function Get_Instantiated_Unit (Target: Iir) return Iir is
   begin
      Check_Kind_For_Instantiated_Unit (Target);
      return Target.Field2;
   end Get_Instantiated_Unit;

   procedure Check_Kind_For_Map_Aspect_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Instantiation_Statement
           | Iir_Kind_Binding_Indication
           | Iir_Kind_Block_Header =>
            null;
         when others =>
            Failed ("map_aspect_list", Target);
      end case;
   end Check_Kind_For_Map_Aspect_List;

   procedure Set_Port_Map_Aspect_List (Target: Iir; Port: Iir_Association_List)
   is
   begin
      Check_Kind_For_Map_Aspect_List (Target);
      Target.Field3 := Port;
   end Set_Port_Map_Aspect_List;

   function Get_Port_Map_Aspect_List (Target: Iir)
     return Iir_Association_List is
   begin
      Check_Kind_For_Map_Aspect_List (Target);
      return Target.Field3;
   end Get_Port_Map_Aspect_List;

   procedure Set_Generic_Map_Aspect_List
     (Target: Iir; Generics: Iir_Association_List) is
   begin
      Check_Kind_For_Map_Aspect_List (Target);
      Target.Field5 := Generics;
   end Set_Generic_Map_Aspect_List;

   function Get_Generic_Map_Aspect_List (Target: Iir)
     return Iir_Association_List is
   begin
      Check_Kind_For_Map_Aspect_List (Target);
      return Target.Field5;
   end Get_Generic_Map_Aspect_List;

   procedure Set_Component_Configuration
     (Target : Iir_Component_Instantiation_Statement;
      Conf : Iir) is
   begin
      Target.Field1 := Conf;
   end Set_Component_Configuration;

   function  Get_Component_Configuration
     (Target : Iir_Component_Instantiation_Statement)
      return Iir
   is
   begin
      return Target.Field1;
   end Get_Component_Configuration;

   procedure Check_Kind_For_Configuration (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Aspect_Configuration =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Configuration;

   procedure Set_Configuration (Target: Iir; Conf : Iir) is
   begin
      Check_Kind_For_Configuration (Target);
      Target.Field1 := Conf;
   end Set_Configuration;

   function Get_Configuration (Target : Iir) return Iir is
   begin
      Check_Kind_For_Configuration (Target);
      return Target.Field1;
   end Get_Configuration;

   procedure Check_Kind_For_Default_Binding_Indication (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Component_Instantiation_Statement =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Default_Binding_Indication;

   procedure Set_Default_Binding_Indication (Target : Iir; Conf : Iir) is
   begin
      Check_Kind_For_Default_Binding_Indication (Target);
      Target.Field4 := Conf;
   end Set_Default_Binding_Indication;

   function Get_Default_Binding_Indication (Target : Iir) return Iir is
   begin
      Check_Kind_For_Default_Binding_Indication (Target);
      return Target.Field4;
   end Get_Default_Binding_Indication;

   procedure Check_Kind_For_Entity_Kind (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Entity_Class
           | Iir_Kind_Attribute_Specification =>
            null;
         when others =>
            Failed;
      end case;
   end Check_Kind_For_Entity_Kind;

   procedure Set_Entity_Kind (Target : Iir;
                              Kind : Tokens.Token_Type) is
   begin
      Check_Kind_For_Entity_Kind (Target);
      Target.Nbr3 := Tokens.Token_Type'Pos (Kind);
   end Set_Entity_Kind;

   function Get_Entity_Kind (Target : Iir) return Tokens.Token_Type is
   begin
      Check_Kind_For_Entity_Kind (Target);
      return Tokens.Token_Type'Val (Target.Nbr3);
   end Get_Entity_Kind;

   function Get_Entity_Name_List (Target : Iir_Attribute_Specification)
                                  return Iir is
   begin
      return Target.Field1;
   end Get_Entity_Name_List;

   procedure Set_Entity_Name_List (Target : Iir_Attribute_Specification;
                                   Names : Iir) is
   begin
      Target.Field1 := Names;
   end Set_Entity_Name_List;

   function Get_Attribute_Designator (Target : Iir_Attribute_Specification)
                                      return Iir is
   begin
      return Target.Field2;
   end Get_Attribute_Designator;

   procedure Set_Attribute_Designator
     (Target: Iir_Attribute_Specification; Designator : Iir) is
   begin
      Target.Field2 := Designator;
   end Set_Attribute_Designator;

   function Get_Attribute_Specification
     (Val : Iir_Attribute_Value) return Iir_Attribute_Specification is
   begin
      return Val.Field2;
   end Get_Attribute_Specification;

   procedure Set_Attribute_Specification
     (Val : Iir_Attribute_Value;
      Attr : Iir_Attribute_Specification) is
   begin
      Val.Field2 := Attr;
   end Set_Attribute_Specification;

   function Get_Designated_Entity (Val : Iir_Attribute_Value) return Iir is
   begin
      return Val.Field3;
   end Get_Designated_Entity;

   procedure Set_Designated_Entity (Val : Iir_Attribute_Value;
                                    Entity : Iir) is
   begin
      Val.Field3 := Entity;
   end Set_Designated_Entity;

   procedure Check_Kind_For_Attribute_Value_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Design_Unit
           | Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Implicit_Procedure_Declaration
           | Iir_Kind_Type_Declaration
           | Iir_Kind_Subtype_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Implicit_Signal_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_File_Declaration
           | Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_Constant_Interface_Declaration
           | Iir_Kind_File_Interface_Declaration
           | Iir_Kind_Component_Declaration
           | Iir_Kind_Group_Declaration
           | Iir_Kind_Enumeration_Literal
           | Iir_Kind_Physical_Unit
           | Iir_Kind_Concurrent_Conditional_Signal_Assignment
           | Iir_Kind_Concurrent_Selected_Signal_Assignment
           | Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Process_Statement
           | Iir_Kind_Concurrent_Assertion_Statement
           | Iir_Kind_Component_Instantiation_Statement
           | Iir_Kind_Block_Statement
           | Iir_Kind_Generate_Statement
           | Iir_Kind_If_Statement
           | Iir_Kind_For_Loop_Statement
           | Iir_Kind_While_Loop_Statement
           | Iir_Kind_Next_Statement
           | Iir_Kind_Exit_Statement
           | Iir_Kind_Signal_Assignment_Statement
           | Iir_Kind_Variable_Assignment_Statement
           | Iir_Kind_Assertion_Statement
           | Iir_Kind_Wait_Statement
           | Iir_Kind_Return_Statement
           | Iir_Kind_Case_Statement
           | Iir_Kind_Procedure_Call
           | Iir_Kind_Concurrent_Procedure_Call_Statement
           | Iir_Kind_Null_Statement
           | Iir_Kind_Attribute_Specification =>
            null;
         when others =>
            Failed ("attribute_value_list", Target);
      end case;
   end Check_Kind_For_Attribute_Value_List;

   function Get_Attribute_Value_List (Target : Iir)
                                      return Iir_Attribute_Value_List is
   begin
      Check_Kind_For_Attribute_Value_List (Target);
      return Target.Attr;
   end Get_Attribute_Value_List;

   procedure Set_Attribute_Value_List (Target : Iir;
                                       List : Iir_Attribute_Value_List) is
   begin
      Check_Kind_For_Attribute_Value_List (Target);
      Target.Attr := List;
   end Set_Attribute_Value_List;

   function Get_Signal_List (Target : Iir_Disconnection_Specification)
     return Iir is
   begin
      return Target.Field2;
   end Get_Signal_List;

   procedure Set_Signal_List (Target : Iir_Disconnection_Specification;
                              List : Iir) is
   begin
      Target.Field2 := List;
   end Set_Signal_List;

   ----------
   -- name --
   ----------

   procedure Check_Kind_For_Name (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Physical_Int_Literal
           | Iir_Kind_Physical_Fp_Literal
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Group_Declaration
           | Iir_Kind_Association_Choice_By_Name =>
            null;
         when others =>
            Failed ("name", Target);
      end case;
   end Check_Kind_For_Name;

   procedure Set_Name (Target: Iir; Name: Iir) is
   begin
      Check_Kind_For_Name (Target);
      Set_Field2 (Target, Name);
   end Set_Name;

   function Get_Name (Target: Iir) return Iir is
   begin
      Check_Kind_For_Name (Target);
      return Get_Field2 (Target);
   end Get_Name;

   procedure Set_Name_Staticness (Target: Iir; Static: Iir_Staticness)
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Indexed_Name
           | Iir_Kind_Slice_Name
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Selected_Element
           | Iir_Kind_Attribute_Value
           | Iir_Kind_Selected_By_All_Name
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference
           | Iir_Kind_Function_Call =>
            null;
         when others =>
            Failed ("set_name_staticness", Target);
      end case;
      Target.Staticness_2 := Static;
   end Set_Name_Staticness;

   function Get_Name_Staticness (Target: Iir) return Iir_Staticness is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Indexed_Name
           | Iir_Kind_Slice_Name
           | Iir_Kind_Alias_Declaration
           | Iir_Kind_Selected_Element
           | Iir_Kind_Attribute_Value
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference
           | Iir_Kind_Function_Call =>
            return Target.Staticness_2;
         when Iir_Kind_Signal_Interface_Declaration
           | Iir_Kind_Signal_Declaration
           | Iir_Kind_Guard_Signal_Declaration
           | Iir_Kind_Variable_Declaration
           | Iir_Kind_Variable_Interface_Declaration
           | Iir_Kind_Constant_Declaration
           | Iir_Kind_Iterator_Declaration
           | Iir_Kind_Constant_Interface_Declaration =>
            -- See 6.1
            return Locally;
         when Iir_Kind_Enumeration_Literal =>
            return Locally;
         when others =>
            Failed ("get_name_staticness", Target);
            return Unknown;
      end case;
   end Get_Name_Staticness;

   ---------------
   -- Attribute --
   ---------------

   procedure Check_Kind_For_Prefix (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Indexed_Name
           | Iir_Kind_Slice_Name
           | Iir_Kind_Parenthesis_Name
           | Iir_Kind_Selected_Name
           | Iir_Kind_Selected_By_All_Name
           | Iir_Kinds_Attribute
           | Iir_Kind_Attribute
           | Iir_Kind_Selected_Element
           | Iir_Kind_Implicit_Dereference
           | Iir_Kind_Dereference =>
            null;
         when others =>
            Failed ("prefix", Target);
      end case;
   end Check_Kind_For_Prefix;

   procedure Set_Prefix (Target: Iir; Prefix: Iir) is
   begin
      Check_Kind_For_Prefix (Target);
      Target.Field3 := Prefix;
   end Set_Prefix;

   function Get_Prefix (Target: Iir) return Iir is
   begin
      Check_Kind_For_Prefix (Target);
      return Target.Field3;
   end Get_Prefix;

   procedure Check_Kind_For_Suffix (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Slice_Name =>
            null;
         when others =>
            Failed ("suffix", Target);
      end case;
   end Check_Kind_For_Suffix;

   procedure Set_Suffix (Target: Iir; Suffix: Iir) is
   begin
      Check_Kind_For_Suffix (Target);
      Set_Field2 (Target, Suffix);
   end Set_Suffix;

   function Get_Suffix (Target: Iir) return Iir is
   begin
      Check_Kind_For_Suffix (Target);
      return Get_Field2 (Target);
   end Get_Suffix;

   procedure Check_Kind_For_Parameter (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Image_Attribute
           | Iir_Kind_Value_Attribute
           | Iir_Kind_Pos_Attribute
           | Iir_Kind_Val_Attribute
           | Iir_Kind_Succ_Attribute
           | Iir_Kind_Pred_Attribute
           | Iir_Kind_Leftof_Attribute
           | Iir_Kind_Rightof_Attribute
           | Iir_Kind_Left_Array_Attribute
           | Iir_Kind_Right_Array_Attribute
           | Iir_Kind_High_Array_Attribute
           | Iir_Kind_Low_Array_Attribute
           | Iir_Kind_Range_Array_Attribute
           | Iir_Kind_Reverse_Range_Array_Attribute
           | Iir_Kind_Length_Array_Attribute
           | Iir_Kind_Ascending_Array_Attribute
           | Iir_Kind_Delayed_Attribute
           | Iir_Kind_Stable_Attribute
           | Iir_Kind_Quiet_Attribute =>
            null;
         when others =>
            Failed ("check_kind_for_parameter", Target);
      end case;
   end Check_Kind_For_Parameter;

   procedure Set_Parameter (Target: Iir; Param: Iir) is
   begin
      Check_Kind_For_Parameter (Target);
      Target.Field2 := Param;
   end Set_Parameter;

   function Get_Parameter (Target: Iir) return Iir is
   begin
      Check_Kind_For_Parameter (Target);
      return Target.Field2;
   end Get_Parameter;

   procedure Check_Kind_For_Association_List (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Parenthesis_Name
           | Iir_Kind_Association_Element_By_Individual  =>
            null;
         when others =>
            Failed ("association_list", Target);
      end case;
   end Check_Kind_For_Association_List;

   procedure Set_Association_List (Target: Iir; List: Iir) is
   begin
      Check_Kind_For_Association_List (Target);
      Target.Field4 := List;
   end Set_Association_List;

   function Get_Association_List (Target: Iir) return Iir is
   begin
      Check_Kind_For_Association_List (Target);
      return Target.Field4;
   end Get_Association_List;

   function Get_Association_Choices_List (Target: Iir)
     return Iir_Association_Choices_List
   is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Aggregate
           | Iir_Kind_Case_Statement =>
            null;
         when others =>
            Failed ("get_association_choices_list", Target);
      end case;
      return Target.Field4;
   end Get_Association_Choices_List;

   procedure Set_Aggregate_Info (Target : Iir_Aggregate;
                                 Info : Iir_Aggregate_Info) is
   begin
      Target.Field2 := Info;
   end Set_Aggregate_Info;

   function Get_Aggregate_Info (Target : Iir_Aggregate)
                               return Iir_Aggregate_Info is
   begin
      return Target.Field2;
   end Get_Aggregate_Info;

   procedure Set_Sub_Aggregate_Info (Target : Iir_Aggregate_Info;
                                     Info : Iir_Aggregate_Info) is
   begin
      Set_Field1 (Target, Info);
   end Set_Sub_Aggregate_Info;

   function Get_Sub_Aggregate_Info (Target : Iir_Aggregate_Info)
                                   return Iir_Aggregate_Info is
   begin
      return Get_Field1 (Target);
   end Get_Sub_Aggregate_Info;

   procedure Set_Aggr_Max_Length (Target : Iir_Aggregate_Info; Nbr : Integer)
   is
   begin
      Target.Nbr2 := Nbr;
   end Set_Aggr_Max_Length;

   function Get_Aggr_Max_Length (Target : Iir_Aggregate_Info) return Integer is
   begin
      return Target.Nbr2;
   end Get_Aggr_Max_Length;

   procedure Set_Aggr_Low_Limit (Target : Iir_Aggregate_Info; Limit : Iir) is
   begin
      Target.Field2 := Limit;
   end Set_Aggr_Low_Limit;

   function Get_Aggr_Low_Limit (Target : Iir_Aggregate_Info) return Iir is
   begin
      return Target.Field2;
   end Get_Aggr_Low_Limit;

   procedure Set_Aggr_High_Limit (Target : Iir_Aggregate_Info; Limit : Iir) is
   begin
      Target.Field3 := Limit;
   end Set_Aggr_High_Limit;

   function Get_Aggr_High_Limit (Target : Iir_Aggregate_Info) return Iir is
   begin
      return Target.Field3;
   end Get_Aggr_High_Limit;

   procedure Set_Aggr_Others_Flag (Target : Iir_Aggregate_Info; Val : Boolean)
   is
   begin
      Target.Flag2 := Val;
   end Set_Aggr_Others_Flag;

   function Get_Aggr_Others_Flag (Target : Iir_Aggregate_Info) return Boolean
   is
   begin
      return Target.Flag2;
   end Get_Aggr_Others_Flag;

   procedure Set_Aggr_Dynamic_Flag (Target : Iir_Aggregate_Info; Val : Boolean)
   is
   begin
      Target.Flag3 := Val;
   end Set_Aggr_Dynamic_Flag;

   function Get_Aggr_Dynamic_Flag (Target : Iir_Aggregate_Info) return Boolean
   is
   begin
      return Target.Flag3;
   end Get_Aggr_Dynamic_Flag;

   procedure Set_Aggr_Named_Flag (Target : Iir_Aggregate_Info; Val : Boolean)
   is
   begin
      Target.Flag4 := Val;
   end Set_Aggr_Named_Flag;

   function Get_Aggr_Named_Flag (Target : Iir_Aggregate_Info) return Boolean is
   begin
      return Target.Flag4;
   end Get_Aggr_Named_Flag;

   procedure Check_Kind_For_Has_Disconnect_Flag (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Signal_Declaration
           | Iir_Kind_Signal_Interface_Declaration =>
            null;
         when others =>
            Error_Kind ("has_disconnect_flag", Target);
      end case;
   end Check_Kind_For_Has_Disconnect_Flag;

   function Get_Has_Disconnect_Flag (Target : Iir) return Boolean is
   begin
      Check_Kind_For_Has_Disconnect_Flag (Target);
      return Get_Flag1 (Target);
   end Get_Has_Disconnect_Flag;

   procedure Set_Has_Disconnect_Flag (Target : Iir; Val : Boolean) is
   begin
      Check_Kind_For_Has_Disconnect_Flag (Target);
      Set_Flag1 (Target, Val);
   end Set_Has_Disconnect_Flag;

   procedure Check_Kind_For_Is_Within_Flag (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Sensitized_Process_Statement
           | Iir_Kind_Process_Statement
           | Iir_Kind_For_Loop_Statement
           | Iir_Kind_Block_Statement
           | Iir_Kind_Entity_Declaration
           | Iir_Kind_Architecture_Declaration
           | Iir_Kind_Function_Specification
           | Iir_Kind_Function_Spec_Body
           | Iir_Kind_Procedure_Specification
           | Iir_Kind_Procedure_Spec_Body
           | Iir_Kind_Implicit_Function_Declaration
           | Iir_Kind_Implicit_Procedure_Declaration =>
            null;
         when others =>
            Error_Kind ("is_within_flag", Target);
      end case;
   end Check_Kind_For_Is_Within_Flag;

   function Get_Is_Within_Flag (Target : Iir) return Boolean is
   begin
      Check_Kind_For_Is_Within_Flag (Target);
      return Get_Flag5 (Target);
   end Get_Is_Within_Flag;

   procedure Set_Is_Within_Flag (Target : Iir; Val : Boolean) is
   begin
      Check_Kind_For_Is_Within_Flag (Target);
      Set_Flag5 (Target, Val);
   end Set_Is_Within_Flag;

   procedure Check_Kind_For_Named_Entity (Target : Iir) is
   begin
      case Get_Kind (Target) is
         when Iir_Kind_Simple_Name
           | Iir_Kind_Selected_Name
           | Iir_Kind_String_Literal
           | Iir_Kind_Parenthesis_Name
           | Iir_Kind_Selected_By_All_Name
           | Iir_Kind_Attribute =>
            null;
         when others =>
            Error_Kind ("named_entity", Target);
      end case;
   end Check_Kind_For_Named_Entity;

   procedure Set_Named_Entity (Target : Iir; Val : Iir) is
   begin
      Check_Kind_For_Named_Entity (Target);
      Set_Field5 (Target, Val);
   end Set_Named_Entity;

   function Get_Named_Entity (Target : Iir) return Iir is
   begin
      Check_Kind_For_Named_Entity (Target);
      return Get_Field5 (Target);
   end Get_Named_Entity;

   function Iir_Predefined_Shortcut_P (Func : Iir_Predefined_Functions)
     return Boolean is
   begin
      case Func is
         when Iir_Predefined_Bit_And
           | Iir_Predefined_Bit_Or
           | Iir_Predefined_Bit_Nand
           | Iir_Predefined_Bit_Nor
           | Iir_Predefined_Boolean_And
           | Iir_Predefined_Boolean_Or
           | Iir_Predefined_Boolean_Nand
           | Iir_Predefined_Boolean_Nor =>
            return True;
         when others =>
            return False;
      end case;
   end Iir_Predefined_Shortcut_P;
end Iirs;
